
// A Command is one line from the logfile that indicates what change to make to the graph.
// When execute() is called it performs the command.

class Command {
  int timestamp;
  String nodeid;
  String action;
  String nodeid2; 
  int connection = -1;
  int sendType = -1;
  boolean isValid;
  String originaltext;
  
  Command(String strline) {
    strline = strline.trim();
    originaltext = strline;
    java.util.StringTokenizer st = new java.util.StringTokenizer(strline);
    if(st.hasMoreTokens()) { st.nextToken(); } //SPEW
    if(st.hasMoreTokens()) { timestamp = stringToInt(st.nextToken()); } //time
    if(st.hasMoreTokens()) { nodeid = st.nextToken(); } //2.0.3.109:2000
    if(st.hasMoreTokens()) { action = st.nextToken().toLowerCase(); } //connect
    if(st.hasMoreTokens()) { nodeid2 = st.nextToken(); } //2.0.3.110:2000
    if(st.hasMoreTokens()) { connection = stringToInt(st.nextToken()); } //1000 or 10000 or 10001
    if(st.hasMoreTokens()) { sendType = stringToInt(st.nextToken()); }  //100 or 102 or 103
    isValid = testValidity();
  } 
  
  //returns true if this is a valid command, false otherwise
  boolean testValidity() {
    boolean test = false;
    
    if(timestamp < 0) { return false; }
    
    if(nodeid == null) { return false; }
    
    if(action.equals("activate")) {
        if(nodeid2 == null && connection == -1) { test = true; }
    } else if(action.equals("connect")) {
        if(nodeid2 != null && connection > -1) { test = true; }
    } else if(action.equals("disconnect")) {
        if(nodeid2 != null && connection > -1) { test = true; }
    } else if(action.equals("sendmessage")) {
        if(nodeid2 != null && connection > -1 && sendType > -1) { test = true; }
    } else if(action.equals("deactivate")) {
        if(nodeid2 == null && connection == -1) { test = true; }
    }
 
    return test;   
  }
  
  //executes the current command
  //returns true if successfull, false the id didn't exist or command not valid
  boolean execute() {
     if(!isValid) { return false; }
     
     //ignore everything haveing to do with host catcher, it is boring
     if(nodeid.equals("1.2.3.4:2000") || (nodeid2 != null && nodeid2.equals("1.2.3.4:2000"))) { return false; }
     
     if(action.equals("activate")) {
        return addNode(nodeid);
    } else if(action.equals("connect")) {
        return connectNodes(nodeid, nodeid2, connection);
    } else if(action.equals("disconnect")) {
        return disconnectNodes(nodeid, nodeid2, connection);
    } else if(action.equals("sendmessage")) {
        return sendMessage(nodeid, nodeid2, connection, sendType);
    } else if(action.equals("deactivate")) {
        return deactivateNode(nodeid);
    }
     
     return false;
  }
  
  //adds a node to the graph
  //returns true if added successfully, false if a node with this id already exists
  boolean addNode(String id) {
    if(graph.getNode(id) != null) { return false; }

    graph.add(new Node(id)); 
    return true;
  }

  //puts a link between nodes
  //returns true if successful, false if either one of the ids doesn't exist
  boolean connectNodes(String id1, String id2, int connectionType) {
     Node node1 = graph.getNode(id1);
     Node node2 = graph.getNode(id2);
     
     if(node1 == null) { addNode(id1); }
     if(node2 == null) { addNode(id2); }
   
     return node1.connect(node2, connectionType);
  }

  //deletes the link between nodes
  //returns true if successful, false if either one of the ids doesn't exist
  boolean disconnectNodes(String id1, String id2, int connectionType) {
     Node node1 = graph.getNode(id1);
     Node node2 = graph.getNode(id2);
   
     if(node1 == null || node2 == null) { return false; }
   
     return node1.disconnect(node2, connectionType);
  }

  //sends a message between the two nodes
  //returns true if successful, false if either one of the ids doesn't exist
  //also returns false if there is no link between the nodes
  boolean sendMessage(String id1, String id2, int connectionType, int send) {
    Node node1 = graph.getNode(id1);
    Node node2 = graph.getNode(id2);
   
    if(node1 == null || node2 == null) { return false; }
  
    return node1.sendMessage(node2, connectionType, send);
  }

  //deletes the node and all of its links
  //returns false if invalid idor if node was already inactive
  boolean deactivateNode(String id) {
     Node node1 = graph.getNode(id);
     if(node1 == null) { return false; }
     return node1.deactivate();
  }

  //returns the integer value of the string.
  int stringToInt(String str) {
    try {
      return (new Integer(str.trim())).intValue();
    } catch(NumberFormatException e) {
     return -1;
    }
  }
  
  
} //end Command class
