class CommandRunner {
  
 
 //this is what determines how fast the simulation runs
 //if dataPerSecond = 10, 10 seconds of sim data will execute in 1 second of real time
 //if dataPerSecond = 1, the simulation will run in real time
 //if dataPerSecond = .1, 1 second of sim data will execute in 10 seconds of real time
 float dataPerSecond = 80;
 
 //this should be low enough to be pleasing,
 //but it's not necessary to redraw at every frame
 //this shouldnt change after program execution
 float secondsUntilRedraw = .1;

 //dont edit these
 int executeTime = 0;
 int frameCounter = 0;
 java.util.ArrayList commands;
 java.util.ArrayList queue;
 long speedUpdateFrame;
 
 CommandRunner(String filename) {
   commands = new java.util.ArrayList(0);
   queue = new java.util.ArrayList(0);
   loadCommands(filename);
   if(commands.size() > 0) {
     executeTime = ((Command)commands.get(0)).timestamp;
   }
 } 
 
//increases the execute rate by a logical multiple
void speedUp() {
   speedUpdateFrame = frameNumber;
   dataPerSecond *= 2.0;
   if(dataPerSecond == 0) { dataPerSecond = .25; }
}

//decreases the execute rate by a logical multiple    
void slowDown() {
   speedUpdateFrame = frameNumber;
   dataPerSecond /= 2.0;
   if(dataPerSecond < .25) { dataPerSecond = 0; }
}

//loads the commands in the log file into the commands array
void loadCommands(String filename) {
   String[] lines = loadStrings(filename);
   
   for(int i = 0; i < lines.length; i++) {
      Command cmd = new Command(lines[i]);
      if(cmd.isValid) { commands.add(cmd); }
      /*else if(cmd.originalText.length() > 0 && cmd.originalText.charAt(0) != '#') {
        println("Unrecognized command: " + cmd.originalText); 
      }*/
   }
}

//this should be called by draw at every tick
void execute() {
  
  int seconds = 0;
  int totalFrames = 0;
  
  //we can draw at least one second of data per frame
  if(dataPerSecond >= framesPerSecond) {
     seconds = (int)(dataPerSecond/framesPerSecond);
  } 
  //otherwise we must skip frames to play data slow enough
  else {
     seconds = 1;
     totalFrames = (int)(1 / (dataPerSecond/framesPerSecond));
     if(frameCounter < totalFrames - 1) {
       frameCounter++;
       runAndRedraw(); 
       return; 
     }
  }
  
  dumbIntoQueue(seconds);
  frameCounter = 0;
  
  runAndRedraw(); 
  
}

//dumps seconds worth of data into the queue (relative to executeTime)
void dumbIntoQueue(int seconds) {
  
  if(commands.size() > 0) {
    Command front = (Command)commands.get(0);
  
    while(front.timestamp < executeTime + seconds) {
       queue.add(front);
       commands.remove(0);
       if(commands.size() == 0) { break; }
       front = (Command)commands.get(0);
     }
    
    executeTime += seconds;
  }
}

//runs the queue and dirties the screen if necessary
void runAndRedraw() {
   if(queue.size() > 0) {
     int framesPerDraw = (int)(secondsUntilRedraw*framesPerSecond);
     
     if(framesPerDraw < 1) { framesPerDraw = 1; }
     
     if(frameNumber % framesPerDraw == 0) {
       runQueueAndClear();
       screenDirty = true;
     } 
   }
}

//executes all commands in the queue and clears it
void runQueueAndClear() {
  for(int i = 0; i < queue.size(); i++) {
   ((Command)queue.get(i)).execute(); 
  }
  queue.clear();
}


}// end CommandRunner class
