
// Graph viewer
// Team SPEW

// Editable Variables                
int window_width = 800;                         // Width of window
int window_height = 600;                        // Height of window
int framesPerSecond = 20;                     // Number of times the screen will be refreshed per second

// Non-editable variables
int xtranslate = 0, ytranslate = 0;
float ztranslate = 1;

Graph graph = new Graph();
CommandRunner cmdrunner;
boolean screenDirty = true;
long frameNumber = 0;

boolean showCircles = false;

int INT_MAX = 99999;
float FLT_MAX = 99999;

boolean transitionMode = false;
int transitionFrame = 0;
int totalTransitionFrames = 10;

// Colors & Fonts
color black = color(0, 0, 0);
color sand = color(221, 221, 204);
color nodeidblack = color(0, 0, 0, 150);
color white = color(250, 250, 250);
color red = color(200, 0, 0);
color green = color(0, 200, 0, 190);
color blue = color(0, 0, 200, 200);
color gray = color(190, 190, 190);
color purple = color(190, 15, 140);
PFont myFont;

///////////////////// BEGIN PROGRAM /////////////////////
//this stuff happens at applet load time
void setup() { 
  size(window_width, window_height);               // window size and 3D  
  framerate(framesPerSecond);                    // initialize window refresh rate
  textFont(loadFont("Verdana-12.vlw"), 12);          // load the font
  cmdrunner = new CommandRunner("sqrt200.log");
}

//the stuff in here happens at every tick
void draw() {
  
    if(transitionMode) {
        graph.nodeTransition();
        transitionFrame++;
        if(transitionFrame > totalTransitionFrames) {
          transitionFrame = 0;
          transitionMode = false;
        }
        screenDirty = true;
    }
  
    //execute commands for this tick
    if(!transitionMode)
        cmdrunner.execute();
    
    //redraw the screen if it has changed
    if(screenDirty) {
      
      //clear the screen
      background(sand);
      if(showCircles) { drawCircles(10); }
      //drawLegend();
      if(frameNumber - cmdrunner.speedUpdateFrame < 50 && frameNumber > 50) {
         drawSimulationSpeed(); 
      }
    
      //translate to correct coordinates
      translate(xtranslate, ytranslate);
    
      //draw links and nodes
      graph.drawLinks();
      graph.drawNodes();
      screenDirty = false;
    }
    
    frameNumber++;
 }
 
//draws the circles for radial mode
void drawCircles(int number) {
  stroke(gray);
  strokeWeight(1);
  fill(sand);
  
   for(int i = number; i >= 0; i--) {
     ellipse(window_width/2+xtranslate, window_height/2+ytranslate, graph.radial_radius*2 * i, graph.radial_radius*2 * i);
   }
}

void drawSimulationSpeed() {
  fill(black);
  text("Simulation Speed = " +  cmdrunner.dataPerSecond + "x", 10, window_height - 10);
}

//draws the text in the corner
void drawLegend() {
  fill(black);
    
  text("Use the arrow keys to scroll", 30, 30);
  //text("Use + and - to zoom", 30, 50);
  text("Press V to toggle node IDs.", 30, 50);
  text("Press Z to slow down, X to speed up the simulation.", 30, 70);
  text("Drag a node to change its position and highlight links.", 30, 90);
}

//listener for keyboard presses
void keyPressed() {
  screenDirty = true;
  if(key == CODED) { 
    if (keyCode == UP) { ytranslate += 30;  } 
    if (keyCode == DOWN) { ytranslate -= 30;  }  
    if (keyCode == LEFT) { xtranslate += 30;  } 
    if (keyCode == RIGHT) { xtranslate -= 30;  } 
  } else {
    if (key == '1') { graph.showAllConnections = !graph.showAllConnections;  } 
    if (key == '=' || key == '+') { graph.radial_radius += 10; graph.radialLayout();  } 
    if (key == '-' || key == '_') { graph.radial_radius -= 10;  if(graph.radial_radius < 10) { graph.radial_radius = 10;  } graph.radialLayout(); }
    if (key == 'v' || key == 'V') { graph.shownodeid = !graph.shownodeid; }
    if (key == 'z' || key == 'Z') { cmdrunner.slowDown(); }
    if (key == 'x' || key == 'X') { cmdrunner.speedUp(); }
    if (key == 'c' || key == 'C') { showCircles = !showCircles; }
  }
}

//this happens when the mouse is pressed
void mousePressed() {
 Node n = graph.getClosestNode(mouseX-xtranslate, mouseY-ytranslate);
 if(n != null && !transitionMode) {
   graph.source = n;
   transitionMode = true;
   graph.radialLayout();
  }
}

//this happens when the mouse is released
void mouseReleased() {  

}
