
// A Command is one line from the logfile that indicates what change to make to the graph.
// When execute() is called it performs the command.

class Command {
  int timestamp;
  String nodeid;
  String action; //activate, connect(param1:id), sendmessage(param1:id, param2:num), disconnect(param1:id), deactivate
  String param1; 
  String param2;
  boolean isValid;
  String originaltext;//////////////////
  
  Command(String strline) {
    strline = strline.trim();
    originaltext = strline;///////////////
    java.util.StringTokenizer st = new java.util.StringTokenizer(strline);
     if(st.hasMoreTokens()) { timestamp = stringToInt(st.nextToken()); }
     if(st.hasMoreTokens()) { nodeid = st.nextToken(); }
     if(st.hasMoreTokens()) { action = st.nextToken().toLowerCase(); }
     if(st.hasMoreTokens()) { param1 = st.nextToken(); }
     if(st.hasMoreTokens()) { param2 = st.nextToken(); }
     isValid = testValidity();
  } 
  
  //returns true if this is a valid command, false otherwise
  boolean testValidity() {
    boolean test = false;
    
    if(timestamp < 0) { return false; }
    
    if(nodeid == null) { return false; }
    
    if(action.equals("activate")) {
        if(param1 == null && param2 == null) { test = true; }
    } else if(action.equals("connect")) {
        if(param1 != null && param2 == null) { test = true; }
    } else if(action.equals("disconnect")) {
        if(param1 != null && param2 == null) { test = true; }
    } else if(action.equals("sendmessage")) {
        if(param1 != null && stringToInt(param2) > -1) { test = true; }
    } else if(action.equals("deactivate")) {
        if(param1 == null && param2 == null) { test = true; }
    }
 
    return test;   
  }
  
  //executes the current command
  //returns true if successfull, false the id didn't exist or command not valid
  boolean execute() {
     if(!isValid) { return false; }
     
     if(action.equals("activate")) {
        return addNode(nodeid);
    } else if(action.equals("connect")) {
        return connectNodes(nodeid, param1);
    } else if(action.equals("disconnect")) {
        return disconnectNodes(nodeid, param1);
    } else if(action.equals("sendmessage")) {
        return sendMessage(nodeid, param1, stringToInt(param2));
    } else if(action.equals("deactivate")) {
        return deactivateNode(nodeid);
    }
     
     return false;
  }
  
  //adds a node to the graph
  //returns true if added successfully, false if a node with this id already exists
  boolean addNode(String id) {
    if(graph.getNode(id) != null) { return false; }

    graph.add(new Node(id)); 
    return true;
  }

  //puts a link between nodes
  //returns true if successful, false if either one of the ids doesn't exist
  boolean connectNodes(String id1, String id2) {
     Node node1 = graph.getNode(id1);
     Node node2 = graph.getNode(id2);
     
     if(node1 == null) { addNode(id1); }
     if(node2 == null) { addNode(id2); }
    
     if(node1 == null || node2 == null) { return false; }
   
     return node1.connect(node2);
  }

  //deletes the link between nodes
  //returns true if successful, false if either one of the ids doesn't exist
  boolean disconnectNodes(String id1, String id2) {
     Node node1 = graph.getNode(id1);
     Node node2 = graph.getNode(id2);
   
     if(node1 == null || node2 == null) { return false; }
   
     return node1.disconnect(node2);
  }

  //sends a message between the two nodes
  //returns true if successful, false if either one of the ids doesn't exist
  //also returns false if there is no link between the nodes
  boolean sendMessage(String id1, String id2, int numMessages) {
    Node node1 = graph.getNode(id1);
    Node node2 = graph.getNode(id2);
   
    if(node1 == null || node2 == null) { return false; }
  
    return node1.sendMessage(node2, numMessages);
  }

  //deletes the node and all of its links
  //returns false if invalid idor if node was already inactive
  boolean deactivateNode(String id) {
     Node node1 = graph.getNode(id);
     if(node1 == null) { return false; }
     return node1.deactivate();
  }

  //returns the integer value of the string.
  int stringToInt(String str) {
    try {
      return (new Integer(str.trim())).intValue();
    } catch(NumberFormatException e) {
     return -1;
    }
  }
  
  
} //end Command class
