import processing.core.*; import java.applet.*; import java.awt.*; import java.awt.image.*; import java.awt.event.*; import java.io.*; import java.net.*; import java.text.*; import java.util.*; import java.util.zip.*; public class graphviewer extends PApplet {
// Graph viewer
// Team SPEW

// Editable Variables                
int window_width = 800;                         // Width of window
int window_height = 600;                        // Height of window
int framesPerSecond = 20;                     // Number of times the screen will be refreshed per second

// Non-editable variables
int xtranslate = 0, ytranslate = 0;
float ztranslate = 1;

Graph graph = new Graph();
CommandRunner cmdrunner;
boolean screenDirty = true;
long frameNumber = 0;

// Colors & Fonts
int black = color(0, 0, 0);
int sand = color(221, 221, 204);
int nodeidblack = color(0, 0, 0, 150);
int white = color(250, 250, 250);
int red = color(200, 0, 0);
int green = color(0, 200, 0, 190);
int blue = color(0, 0, 200, 200);
int gray = color(190, 190, 190);
int purple = color(190, 15, 140);
PFont myFont;

///////////////////// BEGIN PROGRAM /////////////////////
//this stuff happens at applet load time
public void setup() { 
  size(window_width, window_height);               // window size and 3D  
  framerate(framesPerSecond);                    // initialize window refresh rate
  textFont(loadFont("Verdana-12.vlw"), 12);          // load the font
  cmdrunner = new CommandRunner("command.log");
}

//the stuff in here happens at every tick
public void draw() {
  
    //execute commands for this tick
    cmdrunner.execute();
    
    //for(int i = 0; i < 1; i++) { graph.arrange(); }
    graph.arrange();
    
    //move a node if its being dragged
    if(graph.selectedNode != null) {
        graph.selectedNode.changePosition(mouseX-xtranslate, mouseY-ytranslate);
    }
    
    //redraw the screen if it has changed
    if(screenDirty) {
      //clear the screen
      background(sand);
      //drawLegend();
    
      //translate to correct coordinates
      translate(xtranslate, ytranslate);
      //translate(xtranslate + (1-ztranslate)*window_width/2, ytranslate + (1-ztranslate)*window_height/2);
      //scale(ztranslate, ztranslate);
    
      //draw links and nodes
      graph.drawLinks();
      graph.drawNodes();
      screenDirty = false;
    }
    
    frameNumber++;
 }

//draws the text in the corner
public void drawLegend() {
  fill(black);
    
  text("Use the arrow keys to scroll", 30, 30);
  //text("Use + and - to zoom", 30, 50);
  text("Press V to toggle node IDs.", 30, 50);
  text("Press Z to slow down, X to speed up the simulation.", 30, 70);
}

//listener for keyboard presses
public void keyPressed() {
  screenDirty = true;
  if(key == CODED) { 
    if (keyCode == UP) { ytranslate += 30;  } 
    if (keyCode == DOWN) { ytranslate -= 30;  }  
    if (keyCode == LEFT) { xtranslate += 30;  } 
    if (keyCode == RIGHT) { xtranslate -= 30;  } 
  } else {
    if (key == '=' || key == '+') { ztranslate += .1f;  } 
    if (key == '-' || key == '_') { ztranslate -= .1f; if(ztranslate < .1f) { ztranslate = .1f; } }
    if (key == 'v' || key == 'V') { graph.shownodeid = !graph.shownodeid; }
    if (key == 'z' || key == 'Z') { cmdrunner.slowDown(); }
    if (key == 'x' || key == 'X') { cmdrunner.speedUp(); }
    if (key == 'r' || key == 'R') { graph.arranges += 20; }
  }
}

//this happens when the mouse is pressed
public void mousePressed() {
 graph.selectedNode = graph.getClosestNode(mouseX-xtranslate, mouseY-ytranslate);
}

//this happens when the mouse is released
public void mouseReleased() {  
  graph.selectedNode = null;
}

public int squaredDistance(int x1, int y1, int x2, int y2) {
  return (x1-x2)*(x1-x2) + (y1-y2)*(y1-y2);
}


// A Command is one line from the logfile that indicates what change to make to the graph.
// When execute() is called it performs the command.

class Command {
  int timestamp;
  String nodeid;
  String action; //activate, connect(param1:id), sendmessage(param1:id, param2:num), disconnect(param1:id), deactivate
  String param1; 
  String param2;
  boolean isValid;
  String originaltext;//////////////////
  
  Command(String strline) {
    strline = strline.trim();
    originaltext = strline;///////////////
    java.util.StringTokenizer st = new java.util.StringTokenizer(strline);
     if(st.hasMoreTokens()) { timestamp = stringToInt(st.nextToken()); }
     if(st.hasMoreTokens()) { nodeid = st.nextToken(); }
     if(st.hasMoreTokens()) { action = st.nextToken().toLowerCase(); }
     if(st.hasMoreTokens()) { param1 = st.nextToken(); }
     if(st.hasMoreTokens()) { param2 = st.nextToken(); }
     isValid = testValidity();
  } 
  
  //returns true if this is a valid command, false otherwise
  public boolean testValidity() {
    boolean test = false;
    
    if(timestamp < 0) { return false; }
    
    if(nodeid == null) { return false; }
    
    if(action.equals("activate")) {
        if(param1 == null && param2 == null) { test = true; }
    } else if(action.equals("connect")) {
        if(param1 != null && param2 == null) { test = true; }
    } else if(action.equals("disconnect")) {
        if(param1 != null && param2 == null) { test = true; }
    } else if(action.equals("sendmessage")) {
        if(param1 != null && stringToInt(param2) > -1) { test = true; }
    } else if(action.equals("deactivate")) {
        if(param1 == null && param2 == null) { test = true; }
    }
 
    return test;   
  }
  
  //executes the current command
  //returns true if successfull, false the id didn't exist or command not valid
  public boolean execute() {
     if(!isValid) { return false; }
     
     if(action.equals("activate")) {
        return addNode(nodeid);
    } else if(action.equals("connect")) {
        return connectNodes(nodeid, param1);
    } else if(action.equals("disconnect")) {
        return disconnectNodes(nodeid, param1);
    } else if(action.equals("sendmessage")) {
        return sendMessage(nodeid, param1, stringToInt(param2));
    } else if(action.equals("deactivate")) {
        return deactivateNode(nodeid);
    }
     
     return false;
  }
  
  //adds a node to the graph
  //returns true if added successfully, false if a node with this id already exists
  public boolean addNode(String id) {
    if(graph.getNode(id) != null) { return false; }

    graph.add(new Node(id)); 
    return true;
  }

  //puts a link between nodes
  //returns true if successful, false if either one of the ids doesn't exist
  public boolean connectNodes(String id1, String id2) {
     Node node1 = graph.getNode(id1);
     Node node2 = graph.getNode(id2);
     
     if(node1 == null) { addNode(id1); }
     if(node2 == null) { addNode(id2); }
    
     if(node1 == null || node2 == null) { return false; }
   
     return node1.connect(node2);
  }

  //deletes the link between nodes
  //returns true if successful, false if either one of the ids doesn't exist
  public boolean disconnectNodes(String id1, String id2) {
     Node node1 = graph.getNode(id1);
     Node node2 = graph.getNode(id2);
   
     if(node1 == null || node2 == null) { return false; }
   
     return node1.disconnect(node2);
  }

  //sends a message between the two nodes
  //returns true if successful, false if either one of the ids doesn't exist
  //also returns false if there is no link between the nodes
  public boolean sendMessage(String id1, String id2, int numMessages) {
    Node node1 = graph.getNode(id1);
    Node node2 = graph.getNode(id2);
   
    if(node1 == null || node2 == null) { return false; }
  
    return node1.sendMessage(node2, numMessages);
  }

  //deletes the node and all of its links
  //returns false if invalid idor if node was already inactive
  public boolean deactivateNode(String id) {
     Node node1 = graph.getNode(id);
     if(node1 == null) { return false; }
     return node1.deactivate();
  }

  //returns the integer value of the string.
  public int stringToInt(String str) {
    try {
      return (new Integer(str.trim())).intValue();
    } catch(NumberFormatException e) {
     return -1;
    }
  }
  
  
} //end Command class

class CommandRunner {
  
 
 //this is what determines how fast the simulation runs
 //if dataPerSecond = 10, 10 seconds of sim data will execute in 1 second of real time
 //if dataPerSecond = 1, the simulation will run in real time
 //if dataPerSecond = .1, 1 second of sim data will execute in 10 seconds of real time
 float dataPerSecond = 5;
 
 //this should be low enough to be pleasing,
 //but it's not necessary to redraw at every frame
 //this shouldnt change after program execution
 float secondsUntilRedraw = .1f;

 //dont edit these
 int executeTime = 0;
 int frameCounter = 0;
 java.util.ArrayList commands;
 java.util.ArrayList queue;
 
 CommandRunner(String filename) {
   commands = new java.util.ArrayList(0);
   queue = new java.util.ArrayList(0);
   loadCommands(filename);
   if(commands.size() > 0) {
     executeTime = ((Command)commands.get(0)).timestamp;
   }
 } 
 
//increases the execute rate by a logical multiple
public void speedUp() {
   dataPerSecond *= 2.0f;
   println("Simulation Speed = " + dataPerSecond + "x");
}

//decreases the execute rate by a logical multiple    
public void slowDown() {
   dataPerSecond /= 2.0f;
   println("Simulation Speed = " + dataPerSecond + "x");
}

//loads the commands in the log file into the commands array
public void loadCommands(String filename) {
   String[] lines = loadStrings(filename);
   
   for(int i = 0; i < lines.length; i++) {
      Command cmd = new Command(lines[i]);
      if(cmd.isValid) { commands.add(cmd); }
      /*else if(cmd.originalText.length() > 0 && cmd.originalText.charAt(0) != '#') {
        println("Unrecognized command: " + cmd.originalText); 
      }*/
   }
}

//this should be called by draw at every tick
public void execute() {
  
  int seconds = 0;
  int totalFrames = 0;
  
  //we can draw at least one second of data per frame
  if(dataPerSecond >= framesPerSecond) {
     seconds = (int)(dataPerSecond/framesPerSecond);
  } 
  //otherwise we must skip frames to play data slow enough
  else {
     seconds = 1;
     totalFrames = (int)(1 / (dataPerSecond/framesPerSecond));
     if(frameCounter < totalFrames - 1) {
       frameCounter++;
       runAndRedraw(); 
       return; 
     }
  }
  
  dumbIntoQueue(seconds);
  frameCounter = 0;
  
  runAndRedraw(); 
  
}

//dumps seconds worth of data into the queue (relative to executeTime)
public void dumbIntoQueue(int seconds) {
  
  if(commands.size() > 0) {
    Command front = (Command)commands.get(0);
  
    while(front.timestamp < executeTime + seconds) {
       queue.add(front);
       commands.remove(0);
       if(commands.size() == 0) { break; }
       front = (Command)commands.get(0);
     }
    
    executeTime += seconds;
  }
}

//runs the queue and dirties the screen if necessary
public void runAndRedraw() {
   if(queue.size() > 0) {
     int framesPerDraw = (int)(secondsUntilRedraw*framesPerSecond);
     
     if(framesPerDraw < 1) { framesPerDraw = 1; }
     
     if(frameNumber % framesPerDraw == 0) {
       runQueueAndClear();
       screenDirty = true;
     } 
   }
}

//executes all commands in the queue and clears it
public void runQueueAndClear() {
  for(int i = 0; i < queue.size(); i++) {
   ((Command)queue.get(i)).execute(); 
  }
  queue.clear();
}


}// end CommandRunner class

// The graph that is displayed by the graph viewer

class Graph {
 
 java.util.Hashtable nodes;
 java.util.Hashtable links;
 Node selectedNode;
 boolean shownodeid;
 java.util.Enumeration graphenum;
 java.util.Enumeration secondenum;
 int arranges;
  
 //constructor
 Graph() {
   nodes = new java.util.Hashtable(200);
   links = new java.util.Hashtable(500);
   shownodeid = true;
 } 
 
 //adds a node to the hashtable
 public void add(Node anode) {
   nodes.put(anode.id, anode); 
   if(nodes.size() == 1) { 
     anode.fixed = true; 
     anode.x = window_width / 2;
     anode.y = window_height / 2;
   }
   arranges = 15;
 }
 
  //adds a link to the hashtable
 public void add(Link alink) {
   String id = alink.node1.id + "-" + alink.node2.id;
   links.put(id, alink); 
 }
 
 //returns a Node object that has the specified id, or null if not found
 public Node getNode(String id) {
  return (Node)nodes.get(id);
 }
 
 //returns the number of nodes
 public int numNodes() {
   return nodes.size();
 }
 
 //draws all of the nodes
 public void drawNodes() {
   for(graphenum = nodes.elements() ; graphenum.hasMoreElements() ;) {
         ((Node)graphenum.nextElement()).drawMe();
     }
 }
 
 //draws all of the links
 public void drawLinks() {
   for(graphenum = links.elements() ; graphenum.hasMoreElements() ;) {
         ((Link)graphenum.nextElement()).drawMe();
     }
 }

//returns the closest node to the (x,y) coordinate passed in
public Node getClosestNode(int x, int y) {
 int tolerance = 200;
 Node testNode, closestNode = null;
 int cloestDistance = tolerance, squaredDist;
 
 for(graphenum = nodes.elements() ; graphenum.hasMoreElements() ;) {
         testNode = (Node)graphenum.nextElement();
         squaredDist = squaredDistance(testNode.x, testNode.y, x, y);
         if(squaredDist < tolerance && squaredDist < cloestDistance) {
            closestNode = testNode;
            cloestDistance = squaredDist;
            }
     }
 
 return closestNode;
}

//performs one iteration of arrange
 public void arrange() {
   if(arranges > 0) {
     for(graphenum = links.elements() ; graphenum.hasMoreElements() ;) {
         Link alink = ((Link)graphenum.nextElement());
         Node node1 = alink.node1; //to
         Node node2 = alink.node2; //from
         
         double vx = node1.x - node2.x;
	 double vy = node1.y - node2.y;
	 double len = Math.sqrt(vx * vx + vy * vy);
	 double f = (alink.len - len) / (len * 3) ;
	 double dx = f * vx;
	 double dy = f * vy;

	 node1.dx += dx;
	 node1.dy += dy;
	 node2.dx += -dx;
	 node2.dy += -dy;
     }
     
     for(graphenum = nodes.elements() ; graphenum.hasMoreElements() ;) {
         Node n1 = ((Node)graphenum.nextElement());
         double dx = 0;
	 double dy = 0;

        for(secondenum = nodes.elements() ; secondenum.hasMoreElements() ;) {
           Node n2 = ((Node)secondenum.nextElement());
            if (n2 == n1) {
		    continue;
		}
		double vx = n1.x - n2.x;
		double vy = n1.y - n2.y;
		double len = vx * vx + vy * vy;
		if (len == 0) {
		    dx += Math.random();
		    dy += Math.random();
		} else if (len < 100*100) {
		    dx += vx / len;
		    dy += vy / len;
		}
        }
        
         double dlen = dx * dx + dy * dy;
	 if (dlen > 0) {
		dlen = Math.sqrt(dlen) / 2;
		n1.dx += dx / dlen;
		n1.dy += dy / dlen;
	    }
     }
     
    for(graphenum = nodes.elements() ; graphenum.hasMoreElements() ;) {
         Node n = ((Node)graphenum.nextElement());
         if (!n.fixed) {
		n.x += Math.max(-5, Math.min(5, n.dx));
		n.y += Math.max(-5, Math.min(5, n.dy));
		//System.out.println("v= " + n.dx + "," + n.dy);
		/*if (n.x < 0) {
		    n.x = 0;
		} else if (n.x > window_width) {
		    n.x = window_width;
		}
		if (n.y < 0) {
		    n.y = 0;
		} else if (n.y > window_height) {
		    n.y = window_height;
		}*/
	    }
	    n.dx /= 2;
	    n.dy /= 2;
      }

     screenDirty= true;
     arranges--;
    }
  
 } //end arrange()
 
} //end Graph class


//a Link object connects two nodes

class Link {
   Node node1;
   Node node2;
   int messages;
   double len = window_height / 4;
   
   //constructor
   Link(Node one, Node two) {
      node1 = one;
      node2 = two;
      messages = 0;
      graph.add(this);
   }
   
   //draws the line between node1 and node2
   public void drawMe() {
      //determine link color
      int redval = messages * 5;
      if (redval > 255) { redval = 255; }
      int linkcolor = color(redval, 0, 0);
      if(redval == 0) {
        if(graph.selectedNode == node1 || graph.selectedNode == node2 ) { linkcolor = black; }
        else { linkcolor = gray; } 
      }
      
      //draw the link
      strokeJoin(ROUND); strokeCap(ROUND); smooth();
      stroke(linkcolor);  strokeWeight(1);  
      beginShape(LINE_STRIP);  vertex(node1.x, node1.y); vertex(node2.x, node2.y);  endShape();
   }
} //end Link class


// A node object represents a point on the graph.
// This will be the individial peer.

class Node {
  
  String id;
  int x, y;
  
  boolean active;
  int nodecolor;
  java.util.Hashtable links = new java.util.Hashtable(10); 
  java.util.Enumeration nodeenum;
  
  double dx, dy;
  boolean fixed = false;
   
   //constructor
   Node(String myid) {
    id = myid;
    nodecolor = color((int)random(0, 255), (int)random(0, 255), (int)random(0, 255));
    x = (int)random(20, window_width-70);
    y = (int)random(100, window_height-20);
    active = true;
   }

   //instatiates a link object and adds it to both node's link arrays
   //returns true is successful, false if anothernode is null or inactive
   public boolean connect(Node anothernode) {
     
     if(anothernode == null || anothernode == this || !anothernode.active) { return false;  }
     
     //make sure that we aren't already connected
      boolean alreadyconnected = false;
      if(links.get(anothernode.id) != null) { alreadyconnected = true; }

      //create a link object and add it to both nodes link arrays
      if(!alreadyconnected) {
       Link alink = new Link(this, anothernode);
       this.links.put(anothernode.id, alink);
       anothernode.links.put(this.id, alink);
      }
      return true;
   }
   
   //deletes the link from this node to anothernode
   //returns true is successful, false if anothernode is null
   public boolean disconnect(Node anothernode) {
     
     if(anothernode == null || anothernode == this) { return false; }
     
     this.links.remove(anothernode.id);
     anothernode.links.remove(this.id);

     return true;
   }
   
   //sends a message to the destination via the link that connects them
   //returns true if successful, false if no link exists
   public boolean sendMessage(Node destination, int numMessages) {
      if(destination == null || !destination.active || 
         destination == this || numMessages < 0) { return false; }
     
      Link thelink = (Link)links.get(destination.id);
      
      if(thelink != null) {
        thelink.messages += numMessages;
        return true;
      }
      
      return false;
   }
   
   //sets active as false and removes all links joining this node
   //returns false is node was already not active.
   public boolean deactivate() {
      if(!active) { return false ; }
      
      //remove the link from the other node's link array
      for(nodeenum = links.keys() ; nodeenum.hasMoreElements() ;) {
         String nodeid = ((String)nodeenum.nextElement());
         graph.getNode(nodeid).links.remove(this.id);
      }
      
      //delete all links in this node
      links.clear();
      
      active = false;
      return true;
   }
   
   //draws the node primative at its (x,y) coordinate
   public void drawMe() {
     if(active) {
       fill(nodeidblack);
        strokeJoin(ROUND); strokeCap(ROUND);  smooth(); 
        stroke(blue); strokeWeight(20); 
        beginShape(POINTS);  vertex(x, y); endShape();
        if(graph.shownodeid) { text(id, x+10, y-10); }
     }
   }
   
   //Changes the position coordinate of the node
   public void changePosition(int x, int y) {
     this.x = x;
     this.y = y;
     screenDirty = true; 
   }
   
   //resets messages for each link stemming from this node to zero.
   public void clearlinks() {
    for(nodeenum = links.elements() ; nodeenum.hasMoreElements() ;) {
         ((Link)nodeenum.nextElement()).messages = 0;
     }
   }
  
} //end Node class
static public void main(String args[]) {   PApplet.main(new String[] { "graphviewer" });}}