
// Graph viewer
// Team SPEW

// Editable Variables                
int window_width = 800;                         // Width of window
int window_height = 600;                        // Height of window
int framesPerSecond = 20;                     // Number of times the screen will be refreshed per second

// Non-editable variables
int xtranslate = 0, ytranslate = 0;
float ztranslate = 1;

Graph graph = new Graph();
CommandRunner cmdrunner;
boolean screenDirty = true;
long frameNumber = 0;

// Colors & Fonts
int black = color(0, 0, 0);
int sand = color(221, 221, 204);
int nodeidblack = color(0, 0, 0, 150);
int white = color(250, 250, 250);
int red = color(200, 0, 0);
int green = color(0, 200, 0, 190);
int blue = color(0, 0, 200, 200);
int gray = color(190, 190, 190);
int purple = color(190, 15, 140);
PFont myFont;

///////////////////// BEGIN PROGRAM /////////////////////
//this stuff happens at applet load time
void setup() { 
  size(window_width, window_height);               // window size and 3D  
  framerate(framesPerSecond);                    // initialize window refresh rate
  textFont(loadFont("Verdana-12.vlw"), 12);          // load the font
  cmdrunner = new CommandRunner("command.log");
}

//the stuff in here happens at every tick
void draw() {
  
    //execute commands for this tick
    cmdrunner.execute();
    
    //for(int i = 0; i < 1; i++) { graph.arrange(); }
    graph.arrange();
    
    //move a node if its being dragged
    if(graph.selectedNode != null) {
        graph.selectedNode.changePosition(mouseX-xtranslate, mouseY-ytranslate);
    }
    
    //redraw the screen if it has changed
    if(screenDirty) {
      //clear the screen
      background(sand);
      //drawLegend();
    
      //translate to correct coordinates
      translate(xtranslate, ytranslate);
      //translate(xtranslate + (1-ztranslate)*window_width/2, ytranslate + (1-ztranslate)*window_height/2);
      //scale(ztranslate, ztranslate);
    
      //draw links and nodes
      graph.drawLinks();
      graph.drawNodes();
      screenDirty = false;
    }
    
    frameNumber++;
 }

//draws the text in the corner
void drawLegend() {
  fill(black);
    
  text("Use the arrow keys to scroll", 30, 30);
  //text("Use + and - to zoom", 30, 50);
  text("Press V to toggle node IDs.", 30, 50);
  text("Press Z to slow down, X to speed up the simulation.", 30, 70);
}

//listener for keyboard presses
void keyPressed() {
  screenDirty = true;
  if(key == CODED) { 
    if (keyCode == UP) { ytranslate += 30;  } 
    if (keyCode == DOWN) { ytranslate -= 30;  }  
    if (keyCode == LEFT) { xtranslate += 30;  } 
    if (keyCode == RIGHT) { xtranslate -= 30;  } 
  } else {
    if (key == '=' || key == '+') { ztranslate += .1;  } 
    if (key == '-' || key == '_') { ztranslate -= .1; if(ztranslate < .1) { ztranslate = .1; } }
    if (key == 'v' || key == 'V') { graph.shownodeid = !graph.shownodeid; }
    if (key == 'z' || key == 'Z') { cmdrunner.slowDown(); }
    if (key == 'x' || key == 'X') { cmdrunner.speedUp(); }
    if (key == 'r' || key == 'R') { graph.arranges += 20; }
  }
}

//this happens when the mouse is pressed
void mousePressed() {
 graph.selectedNode = graph.getClosestNode(mouseX-xtranslate, mouseY-ytranslate);
}

//this happens when the mouse is released
void mouseReleased() {  
  graph.selectedNode = null;
}

int squaredDistance(int x1, int y1, int x2, int y2) {
  return (x1-x2)*(x1-x2) + (y1-y2)*(y1-y2);
}
