
// A node object represents a point on the graph.
// This will be the individial peer.

class Node {
  
  String id;
  float beforex, beforey;
  float x, y;
  float afterx, aftery;
  
  boolean active;
  color nodecolor;
  java.util.Hashtable links = new java.util.Hashtable(10); 
  java.util.Enumeration nodeenum;
  
  //radial varaibles
  Node parent = null;
  int numLeaves;
  int d = INT_MAX;
   
   //constructor
   Node(String myid) {
    id = myid;
    nodecolor = color((int)random(0, 255), (int)random(0, 255), (int)random(0, 255));
    x = FLT_MAX;
    y = FLT_MAX;
    beforex = FLT_MAX;
    beforey = FLT_MAX;
    afterx = FLT_MAX;
    aftery = FLT_MAX;
    active = true;
   }

   //instatiates a link object and adds it to both node's link arrays
   //returns true is successful, false if anothernode is null or inactive
   boolean connect(Node anothernode, int connectionType) {
     
     if(anothernode == null || anothernode == this || !anothernode.active) { return false;  }
     
     //make sure that we aren't already connected
      boolean alreadyconnected = false;
      if(links.get(anothernode.id + "-" + connectionType) != null) { alreadyconnected = true; }

      //create a link object and add it to both nodes link arrays
      if(!alreadyconnected) {
       Link alink = new Link(this, anothernode, connectionType);
       this.links.put(anothernode.id + "-" + connectionType, alink);
       anothernode.links.put(this.id + "-" + connectionType, alink);
       graph.radialLayout();
      }
      return true;
   }
   
   //deletes the link from this node to anothernode
   //returns true is successful, false if anothernode is null
   boolean disconnect(Node anothernode, int connectionType) {
     
     if(anothernode == null || anothernode == this) { return false; }
     
     this.links.remove(anothernode.id + "-" + connectionType);
     anothernode.links.remove(this.id + "-" + connectionType);

     return true;
   }
   
   //sends a message to the destination via the link that connects them
   //returns true if successful, false if no link exists
   boolean sendMessage(Node destination, int connectionType, int sendType) {
      if(destination == null || !destination.active || destination == this) { return false; }
     
      Link thelink = (Link)links.get(destination.id + "-" + connectionType);
      
      if(thelink != null) {
        thelink.messages += sendType;
        return true;
      }
      
      return false;
   }
   
   //sets active as false and removes all links joining this node
   //returns false is node was already not active.
   boolean deactivate() {
      if(!active) { return false ; }
      
      //remove the link from the other node's link array
      for(nodeenum = links.keys() ; nodeenum.hasMoreElements() ;) {
         String linkname = ((String)nodeenum.nextElement());
         String nodeid = linkname.substring(0, linkname.indexOf('-'));
         graph.getNode(nodeid).links.remove(this.id);
      }
      
      //delete all links in this node
      links.clear();
      
      active = false;
      return true;
   }
   
   //draws the node primative at its (x,y) coordinate
   void drawMe() {
     if(active) {
       String displayid = id.substring(0, id.indexOf(':'));
        fill(nodeidblack);
        strokeJoin(ROUND); strokeCap(ROUND);  smooth(); 
        stroke(blue); strokeWeight(20); 
        beginShape(POINTS);  vertex(x, y); endShape();
        if(graph.shownodeid) { text(displayid, x+10, y-10); }
     }
   }
   
   //Changes the position coordinate of the node
   void changePosition(float newx, float newy) {
     this.x = newx;
     this.y = newy;
     screenDirty = true; 
   }
   
   //resets messages for each link stemming from this node to zero.
   //this really doesnt do anything yet
   void clearlinks() {
    for(nodeenum = links.elements() ; nodeenum.hasMoreElements() ;) {
         ((Link)nodeenum.nextElement()).messages = 0;
     }
   }
   
   
 //counts the number of children that this node has in the BFS tree
 int getNumberOfChildren() {
   int numChildren = 0;
   Link l;
   Node child;
   
   //count children
  for(nodeenum = links.elements() ; nodeenum.hasMoreElements() ;) {
         l = (Link)nodeenum.nextElement();
         if(l.connectionType == 1000) {
           if(l.node1 == this) { child = l.node2; }
           else { child = l.node1; }
           if(child.parent == this) { //it really is a child
             numChildren++;
           }
         }
    }
    
   return numChildren;
 }
   
 //recurses on all children to find number of leaves
 int getNumberOfLeaves() {
  int leaves = 0;
  Link l;
  Node child;  
  
  numLeaves = 0;
 
   if(getNumberOfChildren() == 0) { numLeaves = 0; return 1;} //we are  a leaf
  
   for(nodeenum = links.elements() ; nodeenum.hasMoreElements() ;) {
         l = (Link)nodeenum.nextElement();
         if(l.connectionType == 1000) {
           if(l.node1 == this) { child = l.node2; }
           else { child = l.node1; }
           if(child.parent == this) { //it really is a child
             numLeaves += child.getNumberOfLeaves();
           }
         }
      }
    
    return numLeaves;
 
 }
 
 
 //places this node on the correct position on its circle, the recurses to child nodes
 void place(float start, float end) {
     float total = end - start;
     int childleaves;
     float fraction;
     Link l;
     Node child;
     
     //place me
     float newx = graph.radial_radius*this.d*cos(start + total/2.0) + window_width/2;
     float newy = graph.radial_radius*this.d*sin(start + total/2.0) + window_height/2;
     if(!transitionMode)
       changePosition(newx, newy);
     else {
       beforex = x; beforey = y;
       afterx = newx; aftery = newy; 
     }
     
     //for each child
      for(nodeenum = links.elements() ; nodeenum.hasMoreElements() ;) {
         l = (Link)nodeenum.nextElement();
         if(l.connectionType == 1000) {
           if(l.node1 == this) { child = l.node2; }
           else { child = l.node1; }
           if(child.parent == this) { //it really is a child
             childleaves = child.numLeaves;
             if(childleaves == 0) { childleaves++; }
             fraction = (float)childleaves / this.numLeaves;
             end = start + fraction * total;
             
             child.place(start, end);
             start = end;
           }
         }
      }
 }
  
} //end Node class
