import processing.core.*; import java.applet.*; import java.awt.*; import java.awt.image.*; import java.awt.event.*; import java.io.*; import java.net.*; import java.text.*; import java.util.*; import java.util.zip.*; public class graphviewer extends PApplet {
// Graph viewer
// Team SPEW

// Editable Variables                
int window_width = 800;                         // Width of window
int window_height = 600;                        // Height of window
int framesPerSecond = 20;                     // Number of times the screen will be refreshed per second

// Non-editable variables
int xtranslate = 0, ytranslate = 0;
float ztranslate = 1;

Graph graph = new Graph();
CommandRunner cmdrunner;
boolean screenDirty = true;
long frameNumber = 0;

boolean showCircles = false;

int INT_MAX = 99999;
float FLT_MAX = 99999;

boolean transitionMode = false;
int transitionFrame = 0;
int totalTransitionFrames = 10;

// Colors & Fonts
int black = color(0, 0, 0);
int sand = color(221, 221, 204);
int nodeidblack = color(0, 0, 0, 150);
int white = color(250, 250, 250);
int red = color(200, 0, 0);
int green = color(0, 200, 0, 190);
int blue = color(0, 0, 200, 200);
int gray = color(190, 190, 190);
int purple = color(190, 15, 140);
PFont myFont;

///////////////////// BEGIN PROGRAM /////////////////////
//this stuff happens at applet load time
public void setup() { 
  size(window_width, window_height);               // window size and 3D  
  framerate(framesPerSecond);                    // initialize window refresh rate
  textFont(loadFont("Verdana-12.vlw"), 12);          // load the font
  cmdrunner = new CommandRunner("sqrt200.log");
}

//the stuff in here happens at every tick
public void draw() {
  
    if(transitionMode) {
        graph.nodeTransition();
        transitionFrame++;
        if(transitionFrame > totalTransitionFrames) {
          transitionFrame = 0;
          transitionMode = false;
        }
        screenDirty = true;
    }
  
    //execute commands for this tick
    if(!transitionMode)
        cmdrunner.execute();
    
    //redraw the screen if it has changed
    if(screenDirty) {
      
      //clear the screen
      background(sand);
      if(showCircles) { drawCircles(10); }
      //drawLegend();
      if(frameNumber - cmdrunner.speedUpdateFrame < 50 && frameNumber > 50) {
         drawSimulationSpeed(); 
      }
    
      //translate to correct coordinates
      translate(xtranslate, ytranslate);
    
      //draw links and nodes
      graph.drawLinks();
      graph.drawNodes();
      screenDirty = false;
    }
    
    frameNumber++;
 }
 
//draws the circles for radial mode
public void drawCircles(int number) {
  stroke(gray);
  strokeWeight(1);
  fill(sand);
  
   for(int i = number; i >= 0; i--) {
     ellipse(window_width/2+xtranslate, window_height/2+ytranslate, graph.radial_radius*2 * i, graph.radial_radius*2 * i);
   }
}

public void drawSimulationSpeed() {
  fill(black);
  text("Simulation Speed = " +  cmdrunner.dataPerSecond + "x", 10, window_height - 10);
}

//draws the text in the corner
public void drawLegend() {
  fill(black);
    
  text("Use the arrow keys to scroll", 30, 30);
  //text("Use + and - to zoom", 30, 50);
  text("Press V to toggle node IDs.", 30, 50);
  text("Press Z to slow down, X to speed up the simulation.", 30, 70);
  text("Drag a node to change its position and highlight links.", 30, 90);
}

//listener for keyboard presses
public void keyPressed() {
  screenDirty = true;
  if(key == CODED) { 
    if (keyCode == UP) { ytranslate += 30;  } 
    if (keyCode == DOWN) { ytranslate -= 30;  }  
    if (keyCode == LEFT) { xtranslate += 30;  } 
    if (keyCode == RIGHT) { xtranslate -= 30;  } 
  } else {
    if (key == '1') { graph.showAllConnections = !graph.showAllConnections;  } 
    if (key == '=' || key == '+') { graph.radial_radius += 10; graph.radialLayout();  } 
    if (key == '-' || key == '_') { graph.radial_radius -= 10;  if(graph.radial_radius < 10) { graph.radial_radius = 10;  } graph.radialLayout(); }
    if (key == 'v' || key == 'V') { graph.shownodeid = !graph.shownodeid; }
    if (key == 'z' || key == 'Z') { cmdrunner.slowDown(); }
    if (key == 'x' || key == 'X') { cmdrunner.speedUp(); }
    if (key == 'c' || key == 'C') { showCircles = !showCircles; }
  }
}

//this happens when the mouse is pressed
public void mousePressed() {
 Node n = graph.getClosestNode(mouseX-xtranslate, mouseY-ytranslate);
 if(n != null && !transitionMode) {
   graph.source = n;
   transitionMode = true;
   graph.radialLayout();
  }
}

//this happens when the mouse is released
public void mouseReleased() {  

}


// A Command is one line from the logfile that indicates what change to make to the graph.
// When execute() is called it performs the command.

class Command {
  int timestamp;
  String nodeid;
  String action;
  String nodeid2; 
  int connection = -1;
  int sendType = -1;
  boolean isValid;
  String originaltext;
  
  Command(String strline) {
    strline = strline.trim();
    originaltext = strline;
    java.util.StringTokenizer st = new java.util.StringTokenizer(strline);
    if(st.hasMoreTokens()) { st.nextToken(); } //SPEW
    if(st.hasMoreTokens()) { timestamp = stringToInt(st.nextToken()); } //time
    if(st.hasMoreTokens()) { nodeid = st.nextToken(); } //2.0.3.109:2000
    if(st.hasMoreTokens()) { action = st.nextToken().toLowerCase(); } //connect
    if(st.hasMoreTokens()) { nodeid2 = st.nextToken(); } //2.0.3.110:2000
    if(st.hasMoreTokens()) { connection = stringToInt(st.nextToken()); } //1000 or 10000 or 10001
    if(st.hasMoreTokens()) { sendType = stringToInt(st.nextToken()); }  //100 or 102 or 103
    isValid = testValidity();
  } 
  
  //returns true if this is a valid command, false otherwise
  public boolean testValidity() {
    boolean test = false;
    
    if(timestamp < 0) { return false; }
    
    if(nodeid == null) { return false; }
    
    if(action.equals("activate")) {
        if(nodeid2 == null && connection == -1) { test = true; }
    } else if(action.equals("connect")) {
        if(nodeid2 != null && connection > -1) { test = true; }
    } else if(action.equals("disconnect")) {
        if(nodeid2 != null && connection > -1) { test = true; }
    } else if(action.equals("sendmessage")) {
        if(nodeid2 != null && connection > -1 && sendType > -1) { test = true; }
    } else if(action.equals("deactivate")) {
        if(nodeid2 == null && connection == -1) { test = true; }
    }
 
    return test;   
  }
  
  //executes the current command
  //returns true if successfull, false the id didn't exist or command not valid
  public boolean execute() {
     if(!isValid) { return false; }
     
     //ignore everything haveing to do with host catcher, it is boring
     if(nodeid.equals("1.2.3.4:2000") || (nodeid2 != null && nodeid2.equals("1.2.3.4:2000"))) { return false; }
     
     if(action.equals("activate")) {
        return addNode(nodeid);
    } else if(action.equals("connect")) {
        return connectNodes(nodeid, nodeid2, connection);
    } else if(action.equals("disconnect")) {
        return disconnectNodes(nodeid, nodeid2, connection);
    } else if(action.equals("sendmessage")) {
        return sendMessage(nodeid, nodeid2, connection, sendType);
    } else if(action.equals("deactivate")) {
        return deactivateNode(nodeid);
    }
     
     return false;
  }
  
  //adds a node to the graph
  //returns true if added successfully, false if a node with this id already exists
  public boolean addNode(String id) {
    if(graph.getNode(id) != null) { return false; }

    graph.add(new Node(id)); 
    return true;
  }

  //puts a link between nodes
  //returns true if successful, false if either one of the ids doesn't exist
  public boolean connectNodes(String id1, String id2, int connectionType) {
     Node node1 = graph.getNode(id1);
     Node node2 = graph.getNode(id2);
     
     if(node1 == null) { addNode(id1); }
     if(node2 == null) { addNode(id2); }
   
     return node1.connect(node2, connectionType);
  }

  //deletes the link between nodes
  //returns true if successful, false if either one of the ids doesn't exist
  public boolean disconnectNodes(String id1, String id2, int connectionType) {
     Node node1 = graph.getNode(id1);
     Node node2 = graph.getNode(id2);
   
     if(node1 == null || node2 == null) { return false; }
   
     return node1.disconnect(node2, connectionType);
  }

  //sends a message between the two nodes
  //returns true if successful, false if either one of the ids doesn't exist
  //also returns false if there is no link between the nodes
  public boolean sendMessage(String id1, String id2, int connectionType, int send) {
    Node node1 = graph.getNode(id1);
    Node node2 = graph.getNode(id2);
   
    if(node1 == null || node2 == null) { return false; }
  
    return node1.sendMessage(node2, connectionType, send);
  }

  //deletes the node and all of its links
  //returns false if invalid idor if node was already inactive
  public boolean deactivateNode(String id) {
     Node node1 = graph.getNode(id);
     if(node1 == null) { return false; }
     return node1.deactivate();
  }

  //returns the integer value of the string.
  public int stringToInt(String str) {
    try {
      return (new Integer(str.trim())).intValue();
    } catch(NumberFormatException e) {
     return -1;
    }
  }
  
  
} //end Command class

class CommandRunner {
  
 
 //this is what determines how fast the simulation runs
 //if dataPerSecond = 10, 10 seconds of sim data will execute in 1 second of real time
 //if dataPerSecond = 1, the simulation will run in real time
 //if dataPerSecond = .1, 1 second of sim data will execute in 10 seconds of real time
 float dataPerSecond = 80;
 
 //this should be low enough to be pleasing,
 //but it's not necessary to redraw at every frame
 //this shouldnt change after program execution
 float secondsUntilRedraw = .1f;

 //dont edit these
 int executeTime = 0;
 int frameCounter = 0;
 java.util.ArrayList commands;
 java.util.ArrayList queue;
 long speedUpdateFrame;
 
 CommandRunner(String filename) {
   commands = new java.util.ArrayList(0);
   queue = new java.util.ArrayList(0);
   loadCommands(filename);
   if(commands.size() > 0) {
     executeTime = ((Command)commands.get(0)).timestamp;
   }
 } 
 
//increases the execute rate by a logical multiple
public void speedUp() {
   speedUpdateFrame = frameNumber;
   dataPerSecond *= 2.0f;
   if(dataPerSecond == 0) { dataPerSecond = .25f; }
}

//decreases the execute rate by a logical multiple    
public void slowDown() {
   speedUpdateFrame = frameNumber;
   dataPerSecond /= 2.0f;
   if(dataPerSecond < .25f) { dataPerSecond = 0; }
}

//loads the commands in the log file into the commands array
public void loadCommands(String filename) {
   String[] lines = loadStrings(filename);
   
   for(int i = 0; i < lines.length; i++) {
      Command cmd = new Command(lines[i]);
      if(cmd.isValid) { commands.add(cmd); }
      /*else if(cmd.originalText.length() > 0 && cmd.originalText.charAt(0) != '#') {
        println("Unrecognized command: " + cmd.originalText); 
      }*/
   }
}

//this should be called by draw at every tick
public void execute() {
  
  int seconds = 0;
  int totalFrames = 0;
  
  //we can draw at least one second of data per frame
  if(dataPerSecond >= framesPerSecond) {
     seconds = (int)(dataPerSecond/framesPerSecond);
  } 
  //otherwise we must skip frames to play data slow enough
  else {
     seconds = 1;
     totalFrames = (int)(1 / (dataPerSecond/framesPerSecond));
     if(frameCounter < totalFrames - 1) {
       frameCounter++;
       runAndRedraw(); 
       return; 
     }
  }
  
  dumbIntoQueue(seconds);
  frameCounter = 0;
  
  runAndRedraw(); 
  
}

//dumps seconds worth of data into the queue (relative to executeTime)
public void dumbIntoQueue(int seconds) {
  
  if(commands.size() > 0) {
    Command front = (Command)commands.get(0);
  
    while(front.timestamp < executeTime + seconds) {
       queue.add(front);
       commands.remove(0);
       if(commands.size() == 0) { break; }
       front = (Command)commands.get(0);
     }
    
    executeTime += seconds;
  }
}

//runs the queue and dirties the screen if necessary
public void runAndRedraw() {
   if(queue.size() > 0) {
     int framesPerDraw = (int)(secondsUntilRedraw*framesPerSecond);
     
     if(framesPerDraw < 1) { framesPerDraw = 1; }
     
     if(frameNumber % framesPerDraw == 0) {
       runQueueAndClear();
       screenDirty = true;
     } 
   }
}

//executes all commands in the queue and clears it
public void runQueueAndClear() {
  for(int i = 0; i < queue.size(); i++) {
   ((Command)queue.get(i)).execute(); 
  }
  queue.clear();
}


}// end CommandRunner class

// The graph that is displayed by the graph viewer

class Graph {
 
 java.util.Hashtable nodes;
 java.util.Hashtable links;
 boolean shownodeid;
 java.util.Enumeration graphenum;
 
 boolean showAllConnections = false;
 
 Node source = null;
 int radial_radius = 80;
  
 //constructor
 Graph() {
   nodes = new java.util.Hashtable(200);
   links = new java.util.Hashtable(500);
   shownodeid = true;
 } 
 
 //adds a node to the hashtable
 public void add(Node anode) {
   nodes.put(anode.id, anode); 
   
   //how to determine original center node?
   if(source == null && !anode.id.equals("1.2.3.4:2000")) { 
     source = anode;
   }
 }
 
  //adds a link to the hashtable
 public void add(Link alink) {
   String id = alink.node1.id + "-" + alink.node2.id + "-" + alink.connectionType;
   links.put(id, alink); 
 }
 
 //returns a Node object that has the specified id, or null if not found
 public Node getNode(String id) {
  return (Node)nodes.get(id);
 }
 
 //returns the number of nodes
 public int numNodes() {
   return nodes.size();
 }
 
 //draws all of the nodes
 public void drawNodes() {
   for(graphenum = nodes.elements() ; graphenum.hasMoreElements() ;) {
         ((Node)graphenum.nextElement()).drawMe();
     }
 }
 
 //draws all of the links
 public void drawLinks() {
   for(graphenum = links.elements() ; graphenum.hasMoreElements() ;) {
         ((Link)graphenum.nextElement()).drawMe();
     }
 }

//returns the closest node to the (x,y) coordinate passed in
//or null if no node within tolerance
public Node getClosestNode(int x, int y) {
 float tolerance = 200;
 Node testNode, closestNode = null;
 float cloestDistance = tolerance, squaredDist;
 
 for(graphenum = nodes.elements() ; graphenum.hasMoreElements() ;) {
         testNode = (Node)graphenum.nextElement();
         squaredDist = squaredDistance(testNode.x, testNode.y, x, y);
         
         if(squaredDist < tolerance && squaredDist < cloestDistance) {
            closestNode = testNode;
            cloestDistance = squaredDist;
            }
     }
 
 return closestNode;
}

public float squaredDistance(float x1, float y1, float x2, float y2) {
  return (x1-x2)*(x1-x2) + (y1-y2)*(y1-y2);
}



public void radialLayout() {
    java.util.ArrayList q = new ArrayList();
    Node u, v;
    Link l;
  
  //initialize
  if(source != null) {
   for(graphenum = nodes.elements() ; graphenum.hasMoreElements() ;) {
          u = (Node)graphenum.nextElement();
          u.d = INT_MAX;
          u.parent = null;
      }
   source.d = 0;

   //perform BFS
   q.add(source);
   while(q.size() > 0) {
      u = (Node)q.remove(0);
      for(graphenum = u.links.elements() ; graphenum.hasMoreElements() ;) {
          l = (Link)graphenum.nextElement();
          if(l.connectionType == 1000) {
             if(l.node1 == u) { v = l.node2; }
             else { v = l.node1; }
             if(v.d == INT_MAX) {
              v.d = u.d + 1;
              v.parent = u;
              q.add(v);
             } 
          }
      }
   }
  
   //palce bfs nodes
   source.getNumberOfLeaves();
   source.place(0, 2*PI);
   //place all nodes with no parent out of the screen
   for(graphenum = nodes.elements() ; graphenum.hasMoreElements() ;) {
          u = (Node)graphenum.nextElement();
          if(u.parent == null && graph.source != u) { u.changePosition(FLT_MAX, FLT_MAX); }
      }
  screenDirty = true;
  //lets get outta here
  }
} //end radialLayout()


//move each node x and y one step toward newx and newy
public void nodeTransition() {
   float xtotal, ytotal, frac = (1.0f*transitionFrame) / totalTransitionFrames;
   Node n;
   
   for(graphenum = nodes.elements() ; graphenum.hasMoreElements() ;) {
          
          n = (Node)graphenum.nextElement();
          xtotal = n.afterx - n.beforex;
          ytotal = n.aftery - n.beforey;
          n.x = n.beforex + frac*xtotal;
          n.y = n.beforey + frac*ytotal;
      }

}




 
} //end Graph class


//a Link object connects two nodes

class Link {
   Node node1;
   Node node2;
   int messages;
   int connectionType;
   
   //constructor
   Link(Node one, Node two, int connectType) {
      node1 = one;
      node2 = two;
      messages = 0;
      connectionType = connectType;
      graph.add(this);
   }
   
   //draws the line between node1 and node2
   public void drawMe() {
     
     if(node1.active && node2.active) {
      
      int linkcolor = black;
      
      if(connectionType == 1000) {linkcolor = red;} //searchconnections
      
      if(connectionType == 1000 && ((node1.parent == node2 || node2.parent == node1) || graph.showAllConnections)) { //only show BFS tree links
        //draw the link
        strokeJoin(ROUND); strokeCap(ROUND); smooth();
        stroke(linkcolor);  strokeWeight(1);  
        beginShape(LINE_STRIP);  vertex(node1.x, node1.y); vertex(node2.x, node2.y);  endShape();
      
      //show label
      /*int xlabel = node2.x - (node2.x - node1.x) / 2;
      int ylabel = node2.y - (node2.y - node1.y) / 2;
      
      fill(black);
      text(connectionType, xlabel, ylabel);*/
      }
     }
   }
} //end Link class


// A node object represents a point on the graph.
// This will be the individial peer.

class Node {
  
  String id;
  float beforex, beforey;
  float x, y;
  float afterx, aftery;
  
  boolean active;
  int nodecolor;
  java.util.Hashtable links = new java.util.Hashtable(10); 
  java.util.Enumeration nodeenum;
  
  //radial varaibles
  Node parent = null;
  int numLeaves;
  int d = INT_MAX;
   
   //constructor
   Node(String myid) {
    id = myid;
    nodecolor = color((int)random(0, 255), (int)random(0, 255), (int)random(0, 255));
    x = FLT_MAX;
    y = FLT_MAX;
    beforex = FLT_MAX;
    beforey = FLT_MAX;
    afterx = FLT_MAX;
    aftery = FLT_MAX;
    active = true;
   }

   //instatiates a link object and adds it to both node's link arrays
   //returns true is successful, false if anothernode is null or inactive
   public boolean connect(Node anothernode, int connectionType) {
     
     if(anothernode == null || anothernode == this || !anothernode.active) { return false;  }
     
     //make sure that we aren't already connected
      boolean alreadyconnected = false;
      if(links.get(anothernode.id + "-" + connectionType) != null) { alreadyconnected = true; }

      //create a link object and add it to both nodes link arrays
      if(!alreadyconnected) {
       Link alink = new Link(this, anothernode, connectionType);
       this.links.put(anothernode.id + "-" + connectionType, alink);
       anothernode.links.put(this.id + "-" + connectionType, alink);
       graph.radialLayout();
      }
      return true;
   }
   
   //deletes the link from this node to anothernode
   //returns true is successful, false if anothernode is null
   public boolean disconnect(Node anothernode, int connectionType) {
     
     if(anothernode == null || anothernode == this) { return false; }
     
     this.links.remove(anothernode.id + "-" + connectionType);
     anothernode.links.remove(this.id + "-" + connectionType);

     return true;
   }
   
   //sends a message to the destination via the link that connects them
   //returns true if successful, false if no link exists
   public boolean sendMessage(Node destination, int connectionType, int sendType) {
      if(destination == null || !destination.active || destination == this) { return false; }
     
      Link thelink = (Link)links.get(destination.id + "-" + connectionType);
      
      if(thelink != null) {
        thelink.messages += sendType;
        return true;
      }
      
      return false;
   }
   
   //sets active as false and removes all links joining this node
   //returns false is node was already not active.
   public boolean deactivate() {
      if(!active) { return false ; }
      
      //remove the link from the other node's link array
      for(nodeenum = links.keys() ; nodeenum.hasMoreElements() ;) {
         String linkname = ((String)nodeenum.nextElement());
         String nodeid = linkname.substring(0, linkname.indexOf('-'));
         graph.getNode(nodeid).links.remove(this.id);
      }
      
      //delete all links in this node
      links.clear();
      
      active = false;
      return true;
   }
   
   //draws the node primative at its (x,y) coordinate
   public void drawMe() {
     if(active) {
       String displayid = id.substring(0, id.indexOf(':'));
        fill(nodeidblack);
        strokeJoin(ROUND); strokeCap(ROUND);  smooth(); 
        stroke(blue); strokeWeight(20); 
        beginShape(POINTS);  vertex(x, y); endShape();
        if(graph.shownodeid) { text(displayid, x+10, y-10); }
     }
   }
   
   //Changes the position coordinate of the node
   public void changePosition(float newx, float newy) {
     this.x = newx;
     this.y = newy;
     screenDirty = true; 
   }
   
   //resets messages for each link stemming from this node to zero.
   //this really doesnt do anything yet
   public void clearlinks() {
    for(nodeenum = links.elements() ; nodeenum.hasMoreElements() ;) {
         ((Link)nodeenum.nextElement()).messages = 0;
     }
   }
   
   
 //counts the number of children that this node has in the BFS tree
 public int getNumberOfChildren() {
   int numChildren = 0;
   Link l;
   Node child;
   
   //count children
  for(nodeenum = links.elements() ; nodeenum.hasMoreElements() ;) {
         l = (Link)nodeenum.nextElement();
         if(l.connectionType == 1000) {
           if(l.node1 == this) { child = l.node2; }
           else { child = l.node1; }
           if(child.parent == this) { //it really is a child
             numChildren++;
           }
         }
    }
    
   return numChildren;
 }
   
 //recurses on all children to find number of leaves
 public int getNumberOfLeaves() {
  int leaves = 0;
  Link l;
  Node child;  
  
  numLeaves = 0;
 
   if(getNumberOfChildren() == 0) { numLeaves = 0; return 1;} //we are  a leaf
  
   for(nodeenum = links.elements() ; nodeenum.hasMoreElements() ;) {
         l = (Link)nodeenum.nextElement();
         if(l.connectionType == 1000) {
           if(l.node1 == this) { child = l.node2; }
           else { child = l.node1; }
           if(child.parent == this) { //it really is a child
             numLeaves += child.getNumberOfLeaves();
           }
         }
      }
    
    return numLeaves;
 
 }
 
 
 //places this node on the correct position on its circle, the recurses to child nodes
 public void place(float start, float end) {
     float total = end - start;
     int childleaves;
     float fraction;
     Link l;
     Node child;
     
     //place me
     float newx = graph.radial_radius*this.d*cos(start + total/2.0f) + window_width/2;
     float newy = graph.radial_radius*this.d*sin(start + total/2.0f) + window_height/2;
     if(!transitionMode)
       changePosition(newx, newy);
     else {
       beforex = x; beforey = y;
       afterx = newx; aftery = newy; 
     }
     
     //for each child
      for(nodeenum = links.elements() ; nodeenum.hasMoreElements() ;) {
         l = (Link)nodeenum.nextElement();
         if(l.connectionType == 1000) {
           if(l.node1 == this) { child = l.node2; }
           else { child = l.node1; }
           if(child.parent == this) { //it really is a child
             childleaves = child.numLeaves;
             if(childleaves == 0) { childleaves++; }
             fraction = (float)childleaves / this.numLeaves;
             end = start + fraction * total;
             
             child.place(start, end);
             start = end;
           }
         }
      }
 }
  
} //end Node class
static public void main(String args[]) {   PApplet.main(new String[] { "graphviewer" });}}