
// A Command is one line from the logfile that indicates what change to make to the graph.
// When execute() is called it performs the command.
class Command {
  int timestamp= -1;
  String nodeid1 = "";
  String action = "";
  String nodeid2 = ""; 
  int connectionType = -1;
  int sendType = -1;
  long bytes = -1;
  String comment = "";
  
  boolean isValid;
  String originaltext;
  
  Command(String strline) {
    strline = strline.trim();
    StringTokenizer st = new StringTokenizer(strline);
    if(st.hasMoreTokens()) { st.nextToken(); }
    if(st.hasMoreTokens()) { timestamp = stringToInt(st.nextToken()); } 
    if(st.hasMoreTokens()) { nodeid1 = st.nextToken(); } 
    if(st.hasMoreTokens()) { action = st.nextToken().toLowerCase(); } 
    if(st.hasMoreTokens()) { nodeid2 = st.nextToken(); } 
    if(st.hasMoreTokens()) { connectionType = stringToInt(st.nextToken()); } 
    if(st.hasMoreTokens()) { sendType = stringToInt(st.nextToken()); }  
    if(st.hasMoreTokens()) { bytes = stringToInt(st.nextToken()); } 
    
    while(st.hasMoreTokens()) {
        comment += st.nextToken() + " ";
    }
    comment = comment.trim();
    
    isValid = testValidity();
    composeOriginalText();
  } 
  
  //use pieces to reconstruct a string suitible for displaying
  //this is used when we break and step
  void composeOriginalText() {
    originaltext = "";
    originaltext += timestamp + " ";
    originaltext += nodeid1 + " ";
    originaltext += action + " ";
    if(action.equals("connect") || action.equals("disconnect")) {
        originaltext += nodeid2 + " ";
        originaltext += connectionType;
    }
    if(action.equals("sendmessage")) {
        originaltext += nodeid2 + " ";
        originaltext += connectionType + " ";
        originaltext += bytes + "bytes ";
        originaltext += "\"" + comment + "\"";
    }
  }
  
  //returns true if this is a valid command, false otherwise
  boolean testValidity() {
    boolean test = false;
    
    if(timestamp < 0) { return false; }
    
    if(nodeid1 == "") { return false; }
    
    if(action.equals("activate")) {
        if(nodeid2 == "" && connectionType == -1) { test = true; }
    } else if(action.equals("connect")) {
        if(nodeid2 != "" && connectionType > -1) { test = true; }
    } else if(action.equals("disconnect")) {
        if(nodeid2 != "" && connectionType > -1) { test = true; }
    } else if(action.equals("sendmessage")) {
        if(nodeid2 != "" && connectionType > -1 && sendType != -1 * INT_MAX && bytes != -1 * INT_MAX) { test = true; }
    } else if(action.equals("deactivate")) {
        if(nodeid2 == "" && connectionType == -1) { test = true; }
    }
 
    return test;   
  }
  
  //executes the current command
  //returns true if successfull, false the id didn't exist or command not valid
  boolean execute() {
     if(!isValid) { return false; }
     
     if(action.equals("activate")) {
        return addNode(nodeid1);
    } else if(action.equals("connect")) {
        return connectNodes(nodeid1, nodeid2, connectionType);
    } else if(action.equals("disconnect")) {
        return disconnectNodes(nodeid1, nodeid2, connectionType);
    } else if(action.equals("sendmessage")) {
        return sendMessage(nodeid1, nodeid2, connectionType, bytes);
    } else if(action.equals("deactivate")) {
        return deactivateNode(nodeid1);
    }
     
     return false;
  }
  
  //adds a node to the graph
  //returns true if added successfully, false if a node with this id already exists
  boolean addNode(String id) {
    if(graph.getNode(id) != null) { return false; }

    graph.add(new Node(id)); 
    return true;
  }

  //puts a link between nodes
  //returns true if successful, false if either one of the ids doesn't exist
  boolean connectNodes(String id1, String id2, int connectionType) {
     Node node1 = graph.getNode(id1);
     Node node2 = graph.getNode(id2);
     
     if(node1 == null || node2 == null) { return false; }
   
     return node1.connect(node2, connectionType);
  }

  //deletes the link between nodes
  //returns true if successful, false if either one of the ids doesn't exist
  boolean disconnectNodes(String id1, String id2, int connectionType) {
     Node node1 = graph.getNode(id1);
     Node node2 = graph.getNode(id2);
   
     if(node1 == null || node2 == null) { return false; }
   
     return node1.disconnect(node2, connectionType);
  }

  //sends a message between the two nodes
  //returns true if successful, false if either one of the ids doesn't exist
  //also returns false if there is no link between the nodes
  boolean sendMessage(String id1, String id2, int connectionType, long bytes) {
    Node node1 = graph.getNode(id1);
    Node node2 = graph.getNode(id2);
   
    if(node1 == null || node2 == null) { return false; }
    
    cmdrunner.displayAdd(this);
  
    return node1.sendMessage(node2, connectionType, bytes);
  }

  //deletes the node and all of its links
  //returns false if invalid idor if node was already inactive
  boolean deactivateNode(String id) {
     Node node1 = graph.getNode(id);
     if(node1 == null) { return false; }
     return node1.deactivate();
  }  
  
} //end Command class
