
class CommandRunner {

 //dont edit these
 int executeTime = 0;
 int frameCounter = 0;
 ArrayList queue = new ArrayList(0);
 ArrayList displayqueue = new ArrayList(0);
 
 BufferedReader gzipReader = null;
 Command tempCommand = null;
 Command breakCommand = null;
 String hostcatcher = "";
 int requestType = 1000;
 int resultType = 1001;
 int firstTimestamp = 0;
 boolean isBreaked = false;
 boolean eof = false;
 boolean fileproblem = false;
 
 Command lastExecuted;
 
CommandRunner(String filename) {

   initLog(filename);
   
   tempCommand = getNextCommand();
   
   if(tempCommand != null) {
      hostcatcher = tempCommand.nodeid1;
      executeTime =  tempCommand.timestamp;
      firstTimestamp = executeTime;
   }
 } 
 
//Returns the speed of the simulation, which is based off of the slider value
//We need this because the slider is linear, but we need to have a controllable range of ~0.01 to 100,000
//I determined these functions with best fit lines using a few data points
float getSimulationSpeed() {
  int sliderVal = speedslider.getValue();
  float speed;
  
  if(sliderVal > 47 && sliderVal < 52) { speed = 1.0; }
  else if(sliderVal < 50) { speed = sliderVal / 50.0; }
  else { speed = 6e-6*pow(1.25, sliderVal+3); } 
  
  //truncate excessive precision
  if(speed < 10) { speed = Math.round(speed * (1 / .01)) / (1 / .01); }
  else { speed = Math.round(speed); }
  
  return speed;
  
}
 
//adds a command to the display
void displayAdd(Command cmd) {
  int maxSize = 13;
  
  //if its a result, add it to the queue and check maxSize
  if(isSearchResultConnection(cmd.connectionType) && displayqueue.size() > 0) {
     displayqueue.add(cmd);
     if(displayqueue.size() > maxSize) {
        displayqueue.remove(1); 
     } 
  }
  
  //if its a request, reset the queue if its a new one, else ignore it
  if(isSearchConnection(cmd.connectionType)) {
      if(displayqueue.size() == 0) { 
        displayqueue.add(cmd); 
        if(breakbox.isChecked()) { isBreaked = true; breakCommand = cmd;
           step._visible = true;
           resume._visible = true;
        }
        return;
      }
     //check to see if its an old or new request 
      Command temp = (Command)displayqueue.get(0);
      if(!temp.comment.equals(cmd.comment)) {
         displayqueue.clear();
         //we need to break and cancel all further cmd processing here.
         if(breakbox.isChecked()) { isBreaked = true;  breakCommand = cmd;
           step._visible = true;
           resume._visible = true;
        }
         displayqueue.add(cmd);
      }
  }
  
}

//draws all the commands on the display
void displayDraw() {
  
  Command cmd;
  String message = "";
  String resultMessage;
  color textColor;
  
  for(int i = 0; i < displayqueue.size(); i++) {
      cmd = (Command)displayqueue.get(i);
      if(isSearchConnection(cmd.connectionType)) {
        message = cmd.nodeid1 + " - \"" + cmd.comment + "\"";
        textColor = orange;
      }
      else {      
        message = "   " + cmd.nodeid1 + " - " + cmd.comment + " (" + bytesToString(cmd.bytes) + ")";
        textColor = blue;
      }
      drawMessage(message, width-250, i*20 + 150, textColor, Franklin);
     }
  
}

//this should be called by draw at every tick
void execute() {
  
  int seconds = 0;
  int totalFrames = 0;
  float dataPerSecond = getSimulationSpeed();
  
  //we can draw at least one second of data per frame
  if(dataPerSecond >= framerate) {
     seconds = (int)(dataPerSecond/framerate);
  } 
  //otherwise we must skip frames to play data slow enough
  else {
     seconds = 1;
     totalFrames = (int)(framerate/dataPerSecond);
     if(frameCounter < totalFrames - 1) {
       frameCounter++;
       return; 
     } else {
       frameCounter = 0;
     }
  }
  
  dumbIntoQueue(seconds);  
  runQueueAndClear();
  
}

//seeks to a timestamp
void seek(int timestamp) {
   
   while(tempCommand != null && tempCommand.timestamp < timestamp) {
     tempCommand = getNextCommand();
     if(tempCommand != null) { queue.add(tempCommand); }
  }
   
  executeTime = tempCommand.timestamp;
  runQueueAndClear();
  isBreaked = true;
  breakCommand = tempCommand;
  step._visible = true;
  resume._visible = true;
}

//Runs exactly one command
void step() {
    
  if(queue.size() > 0) {
      breakCommand = (Command)queue.get(0);
      queue.remove(0);
      breakCommand.execute();
      lastExecuted = breakCommand;
  } else {
      breakCommand = getNextCommand();
      if(breakCommand != null) { breakCommand.execute();  lastExecuted = breakCommand; }
  }
}

//dumps seconds worth of data into the queue (relative to executeTime)
//if executeTime is -1 then relative to first command's timestamp
void dumbIntoQueue(int seconds) {
  
    seconds *= 1000; //turn seconds into milliseconds for reading the log
    
    if(tempCommand != null && tempCommand.timestamp < executeTime + seconds) {
      if(tempCommand.nodeid1 != hostcatcher) {
        queue.add(tempCommand);
      }
    }
    else if(tempCommand != null) {
      executeTime += seconds;
      return;
    }
    
    while(true) {
      tempCommand = getNextCommand();
      if(tempCommand != null && tempCommand.timestamp < executeTime + seconds) {
        queue.add(tempCommand);
      }
      else
        break;
    }
    
    executeTime += seconds;
    
}

//executes all commands in the queue and clears it
void runQueueAndClear() { 
  while(queue.size() > 0) {
      breakCommand = (Command)queue.get(0);
      queue.remove(0);
      if(!(breakCommand.nodeid1.equals(hostcatcher) || (breakCommand.nodeid2 != "" && breakCommand.nodeid2.equals(hostcatcher))))
        breakCommand.execute();
      lastExecuted = breakCommand;
      if(isBreaked) { return; }
  }
}

//returns the next valid command that is in the log file.
Command getNextCommand() {
  Command cmd = null;
  String line;
  
 if(gzipReader != null) {
   try {
         if(!gzipReader.ready()) { 
           gzipReader.close(); 
           eof = true;
           return null;
          }
         while(cmd == null || !cmd.isValid || (cmd.nodeid1.equals(hostcatcher) || (cmd.nodeid2 != "" && cmd.nodeid2.equals(hostcatcher)))) {
           line = gzipReader.readLine();
           if(line != null) { cmd = new Command(line); } 
           else {
              eof = true;
              return null; 
           }
         }
     } catch (FileNotFoundException fnfe) {
         fileproblem = true;
      } catch (IOException ioe) {
         fileproblem = true;
      } 
 }
  
  return cmd;
 
}

//initializes the log file input stream
void initLog(String filename) {
  InputStream filestream = openStream(filename);
  
   // use BufferedReader to get one line at a time
      try {
         gzipReader = new BufferedReader(
               new InputStreamReader(
                new GZIPInputStream(filestream)));
   
     } catch (NullPointerException npe) {
        fileproblem = true;
     } catch (FileNotFoundException fnfe) {
         fileproblem = true;
      } catch (IOException ioe) {
         fileproblem = true;
      }
    
}

}// end CommandRunner class
