
// The graph that is displayed by the graph viewer
class Graph {
 
 Hashtable nodes = new Hashtable(400);
 Node source = null;
 boolean radialDirty = true;
 long maxLinkBytes;
 long minLinkBytes;
 long maxNodeBytes;
 long minNodeBytes;

 //constructor
 Graph() {

 } 
 
 //adds a node to the hashtable
 void add(Node anode) {
   nodes.put(anode.id, anode); 
   
   //how to determine original center node?
   if(source == null) {
     source = anode;
   }
 }
 
 //returns a Node object that has the specified id, or null if not found
 Node getNode(String id) {
  return (Node)nodes.get(id);
 }
 
 //returns the radius of the radial circles, based on the position of the zoomslider
 float getRadialRadius() {
    int min = 10, max = 275;
    
    float frac = zoomslider.getValue() / 100.0;
    
    return frac*(max-min) + min;
 }
 
 //draws all of the nodes
 void drawNodes() {
   Enumeration nodeenum;
   Enumeration linkenum;
   long bytes;
   Node node;
   Link link;
   
   strokeJoin(ROUND); strokeCap(ROUND);
   fill(white);
   textAlign(LEFT);
   
   //compute maximum and minimum link bytes
   maxLinkBytes = -1;
   minLinkBytes = -1;
   maxNodeBytes = -1;
   minNodeBytes = -1;
   for(nodeenum = nodes.elements() ; nodeenum.hasMoreElements() ;) {
       node = (Node)nodeenum.nextElement(); 
       bytes = node.uplink + node.downlink;
       if(bytes > maxNodeBytes || maxNodeBytes == -1)
         maxNodeBytes = bytes;
       if(bytes < minNodeBytes || minNodeBytes == -1)
         minNodeBytes = bytes;
       for(linkenum = node.links.elements() ; linkenum.hasMoreElements() ;) {
         link = (Link)linkenum.nextElement();
         bytes = link.bytes;
         if(isSearchConnection(link.connectionType)) {
           if(bytes > maxLinkBytes || maxLinkBytes == -1)
             maxLinkBytes = bytes;
           if(bytes < minLinkBytes || minLinkBytes == -1)
             minLinkBytes = bytes;
         }
       }
     }
   
   //draw the nodes
   for(nodeenum = nodes.elements() ; nodeenum.hasMoreElements() ;) {
         node = ((Node)nodeenum.nextElement());
         node.drawMe();
     }
 }
 
 //draws all of the links
 void drawLinks() {
     Enumeration nodeenum;
     Enumeration linkenum;
     Node node;
     Link link;
     
     if(linksbox.isChecked() || transitionMode || graph.nodes.size() > 100) { noSmooth(); }
     else { smooth(); }
     strokeJoin(ROUND); strokeCap(ROUND);
     strokeWeight(1); 
     
     for(nodeenum = nodes.elements() ; nodeenum.hasMoreElements() ;) {
       node = (Node)nodeenum.nextElement(); 
       for(linkenum = node.links.elements() ; linkenum.hasMoreElements() ;) {
         link = (Link)linkenum.nextElement();
         link.drawMe();
       }
     }
 }
 
 //returns the closest node to the (x,y) coordinate passed in
 //or null if no node within tolerance
 Node getClosestNode(int x, int y) {
  Enumeration nodeenum;
  Node n, closest = null;
  float dist, min = FLT_MAX;
  
  //return null if mouse is in the infobox
  if(x + xtranslate > 735) { return null; }
  
  //fidn the closest node
  for(nodeenum = nodes.elements() ; nodeenum.hasMoreElements() ;) {
          n = (Node)nodeenum.nextElement();
          dist = distance(n.getX(), n.getY(), x, y);
          
          if(dist < min) {
            min = dist;
            closest = n;
          }
      }
      
  //if mouse is outside the bounds of closest return null
  if(closest != null && distance(closest.getX(), closest.getY(), x, y) > closest.getDiameter()/2.0)
    return null;
  
  return closest;
 } 
 
 //returns the node matching the string in the find text box.
 Node getClosestNode(String nodeid) {
  Enumeration nodeenum;
  Node testNode = null;
  
  for(nodeenum = nodes.elements() ; nodeenum.hasMoreElements() ;) {
          testNode = (Node)nodeenum.nextElement();
          if(testNode.id.indexOf(nodeid) == 0) { return testNode; }
      }
  
  return null;
 } 
 
 //resets maxBytes, bytes for links, and uplink and downlink for all nodes
 void resetStats() { 
   Node node;
   Link link;
   Enumeration nodeenum;
   Enumeration linkenum;
   
   maxLinkBytes = 0;
   maxNodeBytes = 0;
   cmdrunner.displayqueue.clear();
     
     for(nodeenum = nodes.elements() ; nodeenum.hasMoreElements() ;) {
       node = (Node)nodeenum.nextElement(); 
       node.uplink = 0;
       node.downlink = 0;
       for(linkenum = node.links.elements() ; linkenum.hasMoreElements() ;) {
         link = (Link)linkenum.nextElement();
         link.bytes = 0;
       }
     }
 }

 //Changes positions of nodes to match correct radial layout with source at the center
 void radialLayout() {
     ArrayList queue = new ArrayList();
     Node iterator, parent, child;
     Link link;
     Enumeration nodeenum;
     Enumeration linkenum;
   
   //initialize
   if(source != null) {
    for(nodeenum = nodes.elements() ; nodeenum.hasMoreElements() ;) {
           iterator = (Node)nodeenum.nextElement();
           iterator.oldd = iterator.d;
           iterator.d = INT_MAX;
           iterator.parent = null;
       }
    source.d = 0;

    //perform BFS
    queue.add(source);
    while(queue.size() > 0) {
       parent = (Node)queue.remove(0);
       for(linkenum = parent.links.elements() ; linkenum.hasMoreElements() ;) {
           link = (Link)linkenum.nextElement();
           if(isSearchConnection(link.connectionType)) {
              child = link.otherNode(parent);
              if(child.d == INT_MAX) {
               child.d = parent.d + 1;
               child.parent = parent;
               queue.add(child);
              } 
           }
       }
    }
   
    //palce bfs nodes
    source.getNumberOfLeaves();
    source.place(0, TWO_PI);
    //place all nodes with no parent out of the screen
    for(nodeenum = nodes.elements() ; nodeenum.hasMoreElements() ;) {
           iterator = (Node)nodeenum.nextElement();
           if(iterator.parent == null && iterator != graph.source) { iterator.d = INT_MAX; }
       }
   //lets get outta here
   }
 } //end radialLayout()
 
 //puts the graphview in transition mode, with node being the new center.
 void beginTransition(Node node) {
  if(node != null && !transitionMode && (source == null || node != source)) {
    source = node; 
    transitionMode = true;
    //computer the new positions
    radialDirty = true;
   } 
 }

} //end Graph class
