
//a Link object connects two nodes
class Link {
   Node node1;
   Node node2;
   int connectionType;
   
   long bytes = 0;
   long lastDrawFrame = -1;
   
   //constructor
   Link(Node one, Node two, int connectType) {
      node1 = one;
      node2 = two;
      connectionType = connectType;
   }
   
   //This will be called by one of the nodes connected by the link
   //it will return the OTHER node, since it could be node1 or node2
   Node otherNode(Node node) {
      if(node == node1) { return node2; }
      if(node == node2) { return node1; }
      return null;
   }
   
   //draws the line between node1 and node2
   void drawMe() {
     
     if(isDisplayLink() && aboveThreshold() && node1.active && node2.active && lastDrawFrame < frameCount) {
      if(isTreeLink() || linksbox.isChecked()) {
        //draw the link
        stroke(getLinkColor());
        
        beginShape(LINE_STRIP);  vertex(node1.getX(), node1.getY()); vertex(node2.getX(), node2.getY());  endShape();
      
        //show label
        float xlabel = node2.getX() - (node2.getX() - node1.getX()) / 2;
        float ylabel = node2.getY() - (node2.getY() - node1.getY()) / 2;
      
        fill(black);
        //text("B:" + bytes, xlabel, ylabel);
        
        lastDrawFrame = frameCount;
      }
     }
   }
 
 //returns true if this link is a search or result connection 
 boolean isDisplayLink() {
   return (isSearchConnection(connectionType) || isSearchResultConnection(connectionType));
 }
 
 //returns true if this link is in the BFS tree
 //NOTE: also returns true for some other important links
 boolean isTreeLink() {
   if(node1.parent == node2 || node2.parent == node1)
     return true;
   if(isSearchResultConnection(connectionType))
     return true;
   if(node1.isPartOfBreakMessage() && node2.isPartOfBreakMessage())
     return true;
   return false;
 }
 
 //returns true if this link is above the link threshold, based on the bytes sent across it.
 boolean aboveThreshold() {
   float frac = 0;
   
   if(threshslider.getValue() == 100) { return true; }
   if(graph.maxLinkBytes == 0) { return true; }
   if(isSearchResultConnection(connectionType)) { return true; }
   if(node1.isPartOfBreakMessage() && node2.isPartOfBreakMessage()) { return true; }
   if(threshslider.getValue() == 0) { return false; }
   
   frac = (1.0*this.bytes) / graph.maxLinkBytes;
   
   return frac >=  1 - (threshslider.getValue() / 100.0);
 }
  
 //returns the link color, based upon this link's bytes compared to graph's maxLinkBytes
 //fades from green -> yellow -> orange -> red 
 color getLinkColor() {
      float frac = 0;
      int maxColor = 240, minColor = 10;
      color linkcolor;
      
      if(isSearchResultConnection(connectionType)) { return blue; }
          
      //get the fraction from minLinkBytes to maxLinkBytes
      if(graph.maxLinkBytes > 0 && (graph.maxLinkBytes-graph.minLinkBytes) > 0) {
        frac = (1.0*this.bytes-graph.minLinkBytes) / (graph.maxLinkBytes-graph.minLinkBytes);
      }
      
      //determine the color
      if(frac < 0.5) {
        linkcolor = color((frac / 0.5) * (maxColor-minColor) + minColor, maxColor, minColor);
      } else {
        linkcolor = color(maxColor, (1.0 - ((frac-.5)/.5)) * (maxColor-minColor) + minColor, minColor); 
      }
     
     return linkcolor;
 }
 
} //end Link class
