
// A node object represents a point on the graph.
// This will be the individial peer.
class Node {
  
  //General variables
  String id;
  boolean active = true;
  long uplink = 0;
  long downlink = 0;
  
  //radial varaibles
  Node parent = null;
  int numLeaves;
  int d = INT_MAX, oldd = INT_MAX;
  float angle, oldangle;
  
  Hashtable links = new Hashtable(50); 
   
   //constructor
   Node(String myid) {
    id = myid;
   }

   //instatiates a link object and adds it to both node's link arrays
   //returns true is successful, false if anothernode is null or inactive
   boolean connect(Node anothernode, int connectionType) {
     
     if(anothernode == null || anothernode == this || !anothernode.active) { return false;  }
     
     //make sure that we aren't already connected
      boolean alreadyconnected = false;
      if(links.get(anothernode.id + "-" + connectionType) != null) { alreadyconnected = true; }

      //create a link object and add it to both nodes link arrays
      if(!alreadyconnected) {
       Link alink = new Link(this, anothernode, connectionType);
       this.links.put(anothernode.id + "-" + connectionType, alink);
       anothernode.links.put(this.id + "-" + connectionType, alink);
       graph.radialDirty = true;
      }
      return true;
   }
   
   
   //sends a message to the destination via the link that connects them
   //returns true if successful
   boolean sendMessage(Node destination, int connectionType, long bytes) {
      //there's a problem, don't do this
      if(destination == null || !destination.active || destination == this) { return false; }
      
      //we want to ignore all uninteresting stuff
      if(!isSearchConnection(connectionType) && !isSearchResultConnection(connectionType)) { return false; } 
      if(id.equals(cmdrunner.hostcatcher) || destination.id.equals(cmdrunner.hostcatcher)) { return false; }
     
      Link thelink = (Link)links.get(destination.id + "-" + connectionType);
      Node otherNode = thelink.otherNode(this);///////null?
      
      if(thelink != null && bytes > 0) {
     
        //update link bandwidth
        thelink.bytes += bytes;
        
        //update my uplink bandwidth
        uplink += bytes;
        
        //update their downlink bandwidth
        otherNode.downlink += bytes;
        
        return true;
      }
      
      return false;
   }
   
   //deletes the link from this node to anothernode
   //returns true is successful, false if anothernode is null
   boolean disconnect(Node anothernode, int connectionType) {
     
     if(anothernode == null || anothernode == this) { return false; }
     
     this.links.remove(anothernode.id + "-" + connectionType);
     anothernode.links.remove(this.id + "-" + connectionType);
     
     graph.radialDirty = true;
     
     return true;
   }
   
   //sets active as false and removes all links joining this node
   //returns false is node was already not active.
   boolean deactivate() {
     Enumeration linkenum;
     
      if(!active) { return false ; }
      
      //remove the link from the other node's link array
      for(linkenum = links.keys() ; linkenum.hasMoreElements() ;) {
         String linkname = ((String)linkenum.nextElement());
         String nodeid = linkname.substring(0, linkname.indexOf('-'));
         String connectionType = linkname.substring(linkname.indexOf('-') + 1);
         graph.getNode(nodeid).links.remove(this.id + "-" + connectionType);
      }
      
      //delete all links in this node
      links.clear();
      
      active = false;
      graph.radialDirty = true;
      return true;
   }
   
   //returns the correct x position of the node with 0 being the center of the graph view.
   float getX() {
       if(!transitionMode)
         return graph.getRadialRadius()*d*cos(angle) + width/2;
       else {
         float oldpos = graph.getRadialRadius()*oldd*cos(oldangle) + width/2;
         float newpos = graph.getRadialRadius()*d*cos(angle) + width/2;
         float total = newpos - oldpos;
         return oldpos + getTransitionFraction()*total; 
       }
   }
   
   //returns the correct y position of the node with 0 being the center of the graph view.
   float getY() {
       if(!transitionMode)
         return graph.getRadialRadius()*d*sin(angle) + height/2;
       else {
         float oldpos = graph.getRadialRadius()*oldd*sin(oldangle) + height/2;
         float newpos = graph.getRadialRadius()*d*sin(angle) + height/2;
         float total = newpos - oldpos;
         return oldpos + getTransitionFraction()*total; 
       }
   }
   
   //draws the node primative at its (x,y) coordinate
   void drawMe() {
     if(active) {
  
        stroke(lightblue);
        
        //see if this node appears in the info box on the right, color appropriately
        Command cmd;
        for(int i = cmdrunner.displayqueue.size() - 1; i >= 0; i--) {
           cmd = (Command)cmdrunner.displayqueue.get(i);
           if(isSearchResultConnection(cmd.connectionType) && cmd.nodeid1.equals(this.id)) {//isResult(cmd.sendType)
               stroke(blue);
           }
           if(isSearchConnection(cmd.connectionType) && cmd.nodeid1.equals(this.id)) {//isRequest(cmd.sendType)
               stroke(orange);
           }
        }
        
        //determine radius 
        strokeWeight(getDiameter());
        
        if(graph.nodes.size() > 100) { noSmooth(); }
        else { smooth(); }  
        beginShape(POINTS);  vertex(getX(), getY()); endShape();
        
        if(ipbox.isChecked() || isPartOfBreakMessage()) { drawMessage(id, getX()+10, getY()-10, white, Franklin); }
     }
   }
   
 //returns true if we are breaked and this node is part of the break message
 boolean isPartOfBreakMessage() {
    if(cmdrunner.isBreaked && cmdrunner.breakCommand != null) { //////////////??
       if(id.equals(cmdrunner.breakCommand.nodeid1))
         return true;
       if(id.equals(cmdrunner.breakCommand.nodeid2))
         return true;
    }
    return false;
 }
  
 //returns the radius of this node based on graph.MaxNodeBytes 
 float getDiameter() {
   float frac = 0;
   int min = 10, max = 40;
   
   if(graph.maxNodeBytes > 0 && (graph.maxNodeBytes-graph.minNodeBytes) > 0) {
        frac = (1.0*(uplink+downlink)-graph.minNodeBytes) / (graph.maxNodeBytes-graph.minNodeBytes);
   }

   return frac*(max-min) + min;
 } 
 
 //counts the number of children that this node has in the BFS tree
 int getNumberOfChildren() {
   Enumeration linkenum;
   int numChildren = 0;
   Link link;
   Node child;
   
   //count children
  for(linkenum = links.elements() ; linkenum.hasMoreElements() ;) {
         link = (Link)linkenum.nextElement();
         if(isSearchConnection(link.connectionType)) {
           child = link.otherNode(this);
           if(child.parent == this) { //it really is a child
             numChildren++;
           }
         }
    }
    
   return numChildren;
 }
   
 //recurses on all children to find number of leaves
 //note: the numLeaves variable is in the score of Node and gets updated.
 int getNumberOfLeaves() {
  Enumeration linkenum;
  int leaves = 0;
  Link link;
  Node child;  
  
  numLeaves = 0;
 
   if(getNumberOfChildren() == 0) { numLeaves = 0; return 1;} //we are  a leaf
  
   for(linkenum = links.elements() ; linkenum.hasMoreElements() ;) {
         link = (Link)linkenum.nextElement();
         if(isSearchConnection(link.connectionType)) {
           child = link.otherNode(this);
           if(child.parent == this) { //it really is a child
             numLeaves += child.getNumberOfLeaves();
           }
         }
      }
    
    return numLeaves;
 
 }
 
 //places this node on the correct position on its circle, the recurses to child nodes
 void place(float start, float end) {
     Enumeration linkenum;
     float total = end - start;
     int childleaves;
     float fraction;
     Link link;
     Node child;
     
     //compute angle for this node
     oldangle = angle;
     angle = start + total/2.0;
     
      //for each child
      for(linkenum = links.elements() ; linkenum.hasMoreElements() ;) {
         link = (Link)linkenum.nextElement();
         
         if(isSearchConnection(link.connectionType)) {
           child = link.otherNode(this);
           if(child.parent == this) { //it really is a child
             childleaves = child.numLeaves;
             if(childleaves == 0) { childleaves++; }
             fraction = (float)childleaves / this.numLeaves;
             end = start + fraction * total;
             
             child.place(start, end);
             start = end;
           }
         }
      }
 }
  
} //end Node class
