import processing.core.*; import mkv.MyGUI.*; import java.applet.*; import java.awt.*; import java.awt.image.*; import java.awt.event.*; import java.io.*; import java.net.*; import java.text.*; import java.util.*; import java.util.zip.*; public class graphviewer extends PApplet {
// Graph viewer
// Team SPEW
// Direct question to eric.withrow@gmail.com



// Editable Variables    
String defaultONAfile = "default.ona.gz";
int framesPerSecond = 20;                     // This is a suggestion only, actual framerate is calculated in variable "framerate"
int totalTransitionFrames = 20;               // The number of frames it takes for the nodes to transition
int initialZoom = 30;                         // The initial zoom (in percentage), min and max values are determined in graph.getRadialRadius()
int xtranslate = -125, ytranslate = 0;        // The initial placement of the center of the graph

// Non-editable variables
Graph graph = new Graph();
CommandRunner cmdrunner;
int INT_MAX = 99999;
float FLT_MAX = 99999;
boolean transitionMode = false;
int transitionFrame = 0;
String ONAfile = getONAfile();

// Colors & Fonts
int black = color(0, 0, 0);
int white = color(250, 250, 250);
int red = color(200, 0, 0);
int green = color(0, 200, 0, 190);
int blue = color(0, 0, 200, 200);
int gray = color(50, 50, 50);
int orange = color(240, 150, 0, 200);
int lightblue = color(66, 114, 204, 200);
int lightgray = color(226,226,226);
int bordergray = color(198,198,198);
PFont Verdana, Franklin;

//GUI variables
MyGUI gui;
MyGUIButton left, right, up, down, center, reset, go, step, resume, advancedmenu;
MyGUICheckBox linksbox, ipbox, pausebox, breakbox;
MyGUIPinSlider zoomslider, speedslider, threshslider;
MyGUITextInput findnode;
MyGUILabel zoomlabel, threshlabel, threshvallabel, speedlabel, speedvallabel;

//Advanced GUI variables
boolean advancedMode = false;
MyGUILabel  a_timestamplabel, a_requestlabel, a_resultlabel, a_hostcatcherlabel;
MyGUICheckBox a_circlesbox;
MyGUITextInput a_timestampinput, a_requestinput, a_resultinput, a_hostcatcherinput;
MyGUIButton a_save, a_timestampgo;

////////////////// Magic Number Methods ////////////////////
  //  1000 is SearchConnection
  //  1001 is SearchResultConnection
  //  1002 is IndexConnection
  // 10000 is HostcatcherRegisterConnection
  // 10001 is HostcatcherQueryConnection
  // generally we've only been interested in SearchConnection.

public boolean isSearchConnection(int ct) {
  return (ct == cmdrunner.requestType);
}

public boolean isSearchResultConnection(int ct) {
  return (ct == cmdrunner.resultType);
}

///////////////////// BEGIN PROGRAM /////////////////////
//this stuff happens at applet load time
public void setup() { 
  size(1000, 680);                  // window size
  framerate(framesPerSecond);       // initialize window refresh rate
  background(black);
  Verdana = loadFont("Verdana-12.vlw");
  Franklin = loadFont("FranklinGothic-Book-11.vlw");
  cmdrunner = new CommandRunner(ONAfile);
  initGUI();
}

//the stuff in here happens at every tick
public void draw() {
  
    //perform the transition, if necessary
    if(transitionMode) {
        transitionFrame++;
        if(transitionFrame > totalTransitionFrames) {
          transitionFrame = 0;
          transitionMode = false;
        }
    }
  
    //execute commands for this tick
    if(!transitionMode && !pausebox.isChecked() && !cmdrunner.isBreaked)
        cmdrunner.execute();
        
    //recompute BFS tree and place nodes
    if(graph.radialDirty) {
       graph.radialLayout();
       graph.radialDirty = false;
    }
    
    //If the GUI is being pressed, execute the right action
    guiPressAction();
    
    //DRAWING STUFF BELOW THIS POINT
    background(black);
    if(a_circlesbox.isChecked()) { drawCircles(15); }
    
    if(cmdrunner.fileproblem && !cmdrunner.eof) { drawMessage("Please make sure " + ONAfile + " is in the data directory and is a valid ONA file.", 10, 22, white, Franklin); }
    
    //translate to correct coordinates and draw links & nodes
    pushMatrix();
    translate(xtranslate, ytranslate);
    graph.drawLinks();
    graph.drawNodes();
    popMatrix();
      
    //draw the stuff related to the GUI on top of the canvas
    placeWidgets();
    drawInfoBox();
 }
 

//draws the info box on the right
public void drawInfoBox() {
  noSmooth();
  stroke(bordergray);
  strokeWeight(2);
  rectMode(CORNER);
  
  //update labels
  String simSpeed = cmdrunner.getSimulationSpeed() + "x";
  if(pausebox.isChecked() && (frameCount/framesPerSecond) % 2 == 0) { simSpeed = "PAUSED"; }
  if(cmdrunner.isBreaked && (frameCount/framesPerSecond) % 2 == 0) { simSpeed = "BREAK"; }
  speedvallabel._text = "Sim Speed = " + simSpeed;
  threshvallabel._text = threshslider.getValue() + "%";
  
  //draw the break box
  fill(lightgray);
  rect(0,height-25,width,25);
  if(cmdrunner.isBreaked && cmdrunner.breakCommand != null) {
    drawMessage("BREAK ON: ", 10, height-8, black, Verdana);
    drawMessage(cmdrunner.breakCommand.originaltext, 80, height-8, black, Franklin);
  } else if(!cmdrunner.eof && cmdrunner.lastExecuted != null) {
      drawMessage("Executed Timestamp: " + cmdrunner.lastExecuted.timestamp + " @ " + fileName(ONAfile), 10, height-8, black, Verdana);
  } else if(cmdrunner.eof) {
    drawMessage("Reached end of " + fileName(ONAfile), 10, height-8, black, Verdana);
  }
  
  //draw the gray rectangle on right over the graph
  fill(lightgray);
  rect(width-265,0,265,height);
  
  if(!advancedMode) {
    if(graph.source != null) {
      drawMessage("Center Node: " + graph.source.id, width-250, 20, black, Verdana);
      drawMessage("   Downlink Transferred: " + bytesToString(graph.source.downlink), width-250, 40, black, Verdana);
      drawMessage("   Uplink Transferred: " + bytesToString(graph.source.uplink), width-250, 60, black, Verdana);
      drawMessage("   Total Transferred: " + bytesToString(graph.source.uplink + graph.source.downlink), width-250, 80, black, Verdana);
      drawMessage("   Number of Neighbors: " + graph.source.getNumberOfChildren(), width-250, 100, black, Verdana);
    }
  
    cmdrunner.displayDraw();  
  } else { //draw the Legend
      smooth();
      drawMessage("========Legend=======", width-240, height-250, black, Verdana);
      stroke(lightblue);
      strokeWeight(10);
      drawMessage("Data Transferred: " + bytesToString(graph.minNodeBytes), width-210, height-216, black, Verdana);
      smooth();
      beginShape(POINTS);  vertex(width-235, height-220); endShape();
      strokeWeight(40);
      drawMessage("Data Transferred: " + bytesToString(graph.maxNodeBytes), width-210, height-171, black, Verdana);
      smooth();
      beginShape(POINTS);  vertex(width-235, height-175); endShape();
      strokeWeight(1);
      stroke(color(10,240,10));
      drawMessage("Data Transferred: " + bytesToString(graph.minLinkBytes), width-240, height-110, black, Verdana);
      smooth();
      beginShape(LINE_STRIP);  vertex(width-250, height-130); vertex(width-140, height-130);  endShape();
      stroke(color(240,10,10));
      drawMessage("Data Transferred: " + bytesToString(graph.maxLinkBytes), width-240, height-60, black, Verdana);
      smooth();
      beginShape(LINE_STRIP);  vertex(width-250, height-80); vertex(width-140, height-80);  endShape();
  }
  
}

//draws the circles for radial mode
public void drawCircles(int number) {
  noFill(); noSmooth(); strokeWeight(1); stroke(gray);
  pushMatrix();
  translate(xtranslate, ytranslate);
  for(int i = number; i >= 0; i--) {
    arc(width/2, height/2, graph.getRadialRadius()*2 * i, graph.getRadialRadius()*2 * i, 0, TWO_PI);
  }
  popMatrix();
}

//draws a message on the screen
public void drawMessage(String message, float x, float y, int textColor, PFont font) {
  textFont(font);
  fill(textColor);
  textMode(MODEL);
  textAlign(LEFT);
  text(message, x, y); 
}

//listener for keyboard presses
public void keyPressed() {

  if(key == CODED) { 
    if (keyCode == UP) {  ytranslate += 30; } 
    if (keyCode == DOWN) { ytranslate -= 30; }  
    if (keyCode == LEFT && !findnode.hasFocus()) { xtranslate += 30; } 
    if (keyCode == RIGHT && !findnode.hasFocus()) { xtranslate -= 30;  } 
  } else {
    if ((keyCode == RETURN || keyCode == ENTER) && findnode.hasFocus()) { goToNode(findnode.getValue()); }
    if ((keyCode == RETURN || keyCode == ENTER) && a_timestampinput.hasFocus()) { goToTimestamp(); }   
    if (key == '=' || key == '+') { zoomslider.setValue(zoomslider.getValue()+3);  } 
    if (key == '-' || key == '_') { zoomslider.setValue(zoomslider.getValue()-3); }
    if (key == 'l' || key == 'L') { linksbox.checked(!linksbox.isChecked());  } 
    if (key == 'z' || key == 'Z') { speedslider.setValue(speedslider.getValue()-1);  }
    if (key == 'x' || key == 'X') { speedslider.setValue(speedslider.getValue()+1);  }
    if (key == 'n' || key == 'N') { threshslider.setValue(threshslider.getValue()-5);  }
    if (key == 'm' || key == 'M') { threshslider.setValue(threshslider.getValue()+5);  }
    if (key == 'c' || key == 'C') { a_circlesbox.checked(!a_circlesbox.isChecked()); }
    if (key == 'o' || key == 'O') { ytranslate = 0; xtranslate = -125; zoomslider.setValue(initialZoom); }
    if (key == 'p' || key == 'P') { pausebox.checked(!pausebox.isChecked()); }
    if (key == 'i' || key == 'I') { ipbox.checked(!ipbox.isChecked()); }
    if (key == 'r' || key == 'R') { graph.resetStats(); }
  }
}

//If the buttons are being held we need to continually perform this action
public void guiPressAction() {   
   if(left.isDragged()) { xtranslate += 30;}
   if(right.isDragged()) { xtranslate -= 30;  }
   if(up.isDragged()) { ytranslate += 30;  }
   if(down.isDragged()) { ytranslate -= 30;  }
}
 
// React if the button object was pressed
public void actionPerformed(ActionEvent e) {
  if(e.getSource() == center) { zoomslider.setValue(initialZoom); ytranslate = 0; xtranslate = -125; }
  if(e.getSource() == reset) { graph.resetStats(); }
  if(e.getSource() == go) { goToNode(findnode.getValue()); }
  if(e.getSource() == step) { if(cmdrunner.isBreaked) { cmdrunner.step(); } }
  if(e.getSource() == advancedmenu) { switchAdvancedMode(); }
  if(e.getSource() == a_timestampgo) { goToTimestamp(); }
  if(e.getSource() == a_save) { 
    cmdrunner.requestType = stringToInt(a_requestinput.getValue());
    cmdrunner.resultType = stringToInt(a_resultinput.getValue());
    cmdrunner.hostcatcher = a_hostcatcherinput.getValue();
  }
  if(e.getSource() == resume) { 
    cmdrunner.isBreaked = false;
    step._visible = false;
    resume._visible = false;
   }
}

//When a person enters a string into the find node box and hits "Go", this happens
public void goToNode(String findtext) {
   Node node = graph.getClosestNode(findtext);
   if(node == null) { 
     if(findnode.getValue().indexOf("Couldn't Find") < 0 && findnode.getValue().indexOf("Enter a node and hit") < 0)
       findnode.setValue("Couldn't Find \"" + findtext + "\""); 
   } else { 
     graph.beginTransition(node);
     findnode.setValue("Enter a node and hit \"go\"");
   }
}

//seeks to the timestamp in the timestamp input box.
//if it is invalid, displays an error.
public void goToTimestamp() {
  String text = a_timestampinput.getValue();
  int timestamp = stringToInt(text);
  
  if(timestamp < cmdrunner.firstTimestamp) {
    a_timestampinput.setValue("Timestamp must be >= " + cmdrunner.firstTimestamp);
  } else {
    graph = new Graph();
    cmdrunner = new CommandRunner(ONAfile);
    cmdrunner.requestType = stringToInt(a_requestinput.getValue());
    cmdrunner.resultType = stringToInt(a_resultinput.getValue());
    cmdrunner.hostcatcher = a_hostcatcherinput.getValue();
    cmdrunner.seek(timestamp);
  }
  
}

//this happens when the mouse is pressed
public void mousePressed() {
   
  //clear out the GoTo box if it has focus
  if(findnode.hasFocus()) 
    if(findnode.getValue().indexOf("Couldn't Find") == 0 || findnode.getValue().indexOf("Enter a node and hit") == 0)
      findnode.setValue("");

  Node node = graph.getClosestNode(mouseX-xtranslate, mouseY-ytranslate);
  graph.beginTransition(node);
}

////////////////// GUI Setup Methods ////////////////////
//Initializes all the gui widgets
public void initGUI() {
  //Set up the GUI
  gui = new MyGUI(this, 50);
  gui.getStyle().setFont(Verdana, 12);
  gui.getStyle().tintColor(white); //mouseover EXCEPT for buttons
  gui.getStyle().buttonText = black; //color of LABEL text
  
  //Normal GUI elements
  breakbox = new MyGUICheckBox(this, width-240, height-225, "Break On Event");
  linksbox = new MyGUICheckBox(this, width-240, height-205, "Show All Links");
  ipbox = new MyGUICheckBox(this, width-115, height-205, "Show All IPs");
  left = new MyGUIButton(this, width-236, height-160, "<", 20, 16);
  center = new MyGUIButton(this, width-210, height-160, "-", 20, 16);
  right = new MyGUIButton(this, width-184, height-160, ">", 20, 16);
  up = new MyGUIButton(this, width-210, height-182, "/\\", 20, 16);
  down = new MyGUIButton(this, width-210, height-138, "\\/", 20, 16);
  zoomslider = new MyGUIPinSlider(this, width-85, height-170, 140, 15, 0, 100);
  zoomlabel = new MyGUILabel(this, "Zoom:", width-156, height-188);
  reset = new MyGUIButton(this, width-85, height-140, "Reset B/W Counters", 140, 16);
  threshslider = new MyGUIPinSlider(this, width-115, height-115, 126, 15, 0, 100);
  threshlabel = new MyGUILabel(this, "Threshold:", width-254, height-115);
  threshvallabel = new MyGUILabel(this, "100%", width-40, height-115);
  speedslider = new MyGUIPinSlider(this, width-105, height-90, 180, 15, 0, 100);
  speedlabel = new MyGUILabel(this, "Speed:", width-250, height-90);
  speedvallabel = new MyGUILabel(this, "Sim Speed = 1.0x", width-170, height-65);
  pausebox = new MyGUICheckBox(this, width-240, height-65, "Pause");
  findnode = new MyGUITextInput(this, width-150, height-40, 200, 20);
  go = new MyGUIButton(this, width-25, height-40, "Go", 30, 20);
  advancedmenu = new MyGUIButton(this, width-130, height-15, "Advanced Options & Legend", 240, 16);
  step = new MyGUIButton(this, width-390, height-13, "Step >", 50, 16);
  resume = new MyGUIButton(this, width-315, height-13, "Resume >>", 80, 16);
  
  gui.add(linksbox); gui.add(ipbox);
  gui.add(left); gui.add(center); gui.add(right); gui.add(up); gui.add(down);
  gui.add(zoomslider); gui.add(zoomlabel);
  gui.add(reset);
  gui.add(threshslider); gui.add(threshlabel); gui.add(threshvallabel);
  gui.add(speedslider); gui.add(speedlabel); gui.add(speedvallabel);
  gui.add(pausebox);
  gui.add(findnode); gui.add(go);
  gui.add(advancedmenu);
  gui.add(breakbox); gui.add(step); gui.add(resume);

  zoomslider.setValue(initialZoom);
  speedslider.setValue(50);
  threshslider.setValue(100);
  findnode.setValue("Enter a node and hit \"go\"");
  step._visible = false;
  resume._visible = false;
  
  //Advanced GUI elements
  a_circlesbox = new MyGUICheckBox(this, width-240, 30, "Show Circles");
  a_timestamplabel = new MyGUILabel(this, "Go to Timestamp:", width-250, 80);
  a_timestampinput = new MyGUITextInput(this, width-150, 100, 200, 20);
  a_timestampgo = new MyGUIButton(this, width-30, 100, "Go", 30, 20);
  a_requestlabel = new MyGUILabel(this, "Request Connection Types:", width-250, 150);
  a_requestinput = new MyGUITextInput(this, width-175, 170, 150, 20);
  a_resultlabel = new MyGUILabel(this, "Result Connection Types:", width-250, 190);
  a_resultinput = new MyGUITextInput(this, width-175, 210, 150, 20);
  a_hostcatcherlabel = new MyGUILabel(this, "Host Catcher:", width-250, 230);
  a_hostcatcherinput = new MyGUITextInput(this, width-175, 250, 150, 20);
  a_save = new MyGUIButton(this, width-220, 280, "Save", 60, 20);
  
  gui.add(a_circlesbox);
  gui.add(a_timestamplabel); gui.add(a_timestampinput); gui.add(a_timestampgo); 
  gui.add(a_requestlabel); gui.add(a_requestinput); gui.add(a_resultlabel); gui.add(a_resultinput); 
  gui.add(a_hostcatcherlabel); gui.add(a_hostcatcherinput); gui.add(a_save);
  
  a_circlesbox.checked(true);
  a_requestinput.setValue(new Integer(cmdrunner.requestType).toString());
  a_resultinput.setValue(new Integer(cmdrunner.resultType).toString());
  a_timestampinput.setValue(new Integer(cmdrunner.firstTimestamp).toString());
  a_hostcatcherinput.setValue(cmdrunner.hostcatcher);
  
  //set initial state
  placeWidgets();
}

//switched the infobar to the advanced menu screen and back.
public void switchAdvancedMode() {
   advancedMode = !advancedMode;
   if(advancedMode) { advancedmenu._text = "Back to Main Menu"; } 
   else { advancedmenu._text = "Advanced Options & Legend"; } 
}

//Determines correct widget state
public void placeWidgets() {
   breakbox._visible = !advancedMode;
   linksbox._visible = !advancedMode; ipbox._visible = !advancedMode;
   left._visible = !advancedMode; center._visible = !advancedMode;  right._visible = !advancedMode;  up._visible = !advancedMode;  down._visible = !advancedMode; 
   zoomslider._visible = !advancedMode; zoomlabel._visible = !advancedMode;
   reset._visible = !advancedMode;
   threshslider._visible = !advancedMode; threshlabel._visible = !advancedMode; threshvallabel._visible = !advancedMode;
   speedslider._visible = !advancedMode; speedlabel._visible = !advancedMode; speedvallabel._visible = !advancedMode;
   pausebox._visible = !advancedMode;
   findnode._visible = !advancedMode; go._visible = !advancedMode;
   reset._visible = !advancedMode;
   
  a_circlesbox._visible = advancedMode;
  a_timestamplabel._visible = advancedMode; a_timestampinput._visible = advancedMode; a_timestampgo._visible = advancedMode; 
  a_requestlabel._visible = advancedMode; a_requestinput._visible = advancedMode; a_resultlabel._visible = advancedMode; a_resultinput._visible = advancedMode; 
  a_hostcatcherlabel._visible = advancedMode; a_hostcatcherinput._visible = advancedMode; a_save._visible = advancedMode;
}

////////////////// Global Util Methods ////////////////////

//returns the percentage of the way through the transition
//from 0 to 1, right now uses slow-in/slow-out transition
public float getTransitionFraction() {
   float stepFraction = (1.0f*transitionFrame) / totalTransitionFrames;
   return atan((stepFraction) * 10 - 5) * 0.5f / atan(5) + 0.5f; 
}

//returns the integer value of the string.
  public int stringToInt(String str) {
    try {
      return (new Integer(str.trim())).intValue();
    } catch(NumberFormatException e) {
     return -1 * INT_MAX;
    }
  }
  
 //takes in a number of bytes and returns it in pretty form, for example 21.4 MB
 public String bytesToString(long bytes) {
        float val;
	String answer;
	
	if(bytes >= pow(2, 30)) {
		val = round(bytes / pow(2, 30) * 100.0f) / 100.0f;
		answer = val + " GiB";
	} else if(bytes >= pow(2, 20)) {
		val = round(bytes / pow(2, 20) * 100.0f) / 100.0f;
		answer = val + " MiB";
        } else if(bytes >= pow(2, 10)) {
		val = round(bytes / pow(2, 10) * 100.0f) / 100.0f;
		answer = val + " KiB";
	} else
		answer = bytes + " B";
	
	return answer;
 }
 
//returns the filename without any path information
public String fileName(String path) {
  File file = new File(path);
  return file.getName();
}
 
 //returns the distance between two points
 public float distance(float x1, float y1, float x2, float y2) {
   return sqrt((x1-x2)*(x1-x2) + (y1-y2)*(y1-y2));
 } 

 
// Don't call this from inside setup.
// application will probably deadlock.
//
// We weren't sure if we should have a file chooser or if using Swing components within Processing (AWT)
// would cause problems.  For that reason, this file choosing code is all contained in this method.
// I don't even use imports - I fully qualify the class names.  So if this method is giving you troubles
// or you just don't need a file chooser, remove this method and the one call to it.  Done!
//  - Chris Sidi
public String getONAfile() {
  try {
    javax.swing.UIManager.setLookAndFeel(javax.swing.UIManager.getSystemLookAndFeelClassName());
  } 
  catch (Exception e) 
  {}
  
  String rv = defaultONAfile;
  
  try {
    SecurityManager security = System.getSecurityManager();
    if (null != security) {
      security.checkRead(".");
    }
    
    javax.swing.filechooser.FileFilter ff = new javax.swing.filechooser.FileFilter()
    {
      public boolean accept(File f) {
          if (f.isDirectory()) {
              return true;
          }

          return f.getName().toLowerCase().endsWith(".ona.gz");
      }
      
      public String getDescription() {
        return "Compressed ONA files (*.ona.gz)";
      }
    };
    
    javax.swing.JFileChooser fc = new javax.swing.JFileChooser(new File("."));
    fc.setFileFilter(ff);
    while (javax.swing.JFileChooser.APPROVE_OPTION != fc.showOpenDialog(null)) { 
      //cancel was pressed here, just return the default
      return defaultONAfile;
    }
  
    rv = fc.getSelectedFile().toString();
  }
  catch (SecurityException ex)
  {
    System.out.println("INFO: Not allowed to launch file dialog.  Defaulting to : "+ defaultONAfile +" ("+ ex +")");  
  }
  
  return rv;
}


// A Command is one line from the logfile that indicates what change to make to the graph.
// When execute() is called it performs the command.
class Command {
  int timestamp= -1;
  String nodeid1 = "";
  String action = "";
  String nodeid2 = ""; 
  int connectionType = -1;
  int sendType = -1;
  long bytes = -1;
  String comment = "";
  
  boolean isValid;
  String originaltext;
  
  Command(String strline) {
    strline = strline.trim();
    StringTokenizer st = new StringTokenizer(strline);
    if(st.hasMoreTokens()) { st.nextToken(); }
    if(st.hasMoreTokens()) { timestamp = stringToInt(st.nextToken()); } 
    if(st.hasMoreTokens()) { nodeid1 = st.nextToken(); } 
    if(st.hasMoreTokens()) { action = st.nextToken().toLowerCase(); } 
    if(st.hasMoreTokens()) { nodeid2 = st.nextToken(); } 
    if(st.hasMoreTokens()) { connectionType = stringToInt(st.nextToken()); } 
    if(st.hasMoreTokens()) { sendType = stringToInt(st.nextToken()); }  
    if(st.hasMoreTokens()) { bytes = stringToInt(st.nextToken()); } 
    
    while(st.hasMoreTokens()) {
        comment += st.nextToken() + " ";
    }
    comment = comment.trim();
    
    isValid = testValidity();
    composeOriginalText();
  } 
  
  //use pieces to reconstruct a string suitible for displaying
  //this is used when we break and step
  public void composeOriginalText() {
    originaltext = "";
    originaltext += timestamp + " ";
    originaltext += nodeid1 + " ";
    originaltext += action + " ";
    if(action.equals("connect") || action.equals("disconnect")) {
        originaltext += nodeid2 + " ";
        originaltext += connectionType;
    }
    if(action.equals("sendmessage")) {
        originaltext += nodeid2 + " ";
        originaltext += connectionType + " ";
        originaltext += bytes + "bytes ";
        originaltext += "\"" + comment + "\"";
    }
  }
  
  //returns true if this is a valid command, false otherwise
  public boolean testValidity() {
    boolean test = false;
    
    if(timestamp < 0) { return false; }
    
    if(nodeid1 == "") { return false; }
    
    if(action.equals("activate")) {
        if(nodeid2 == "" && connectionType == -1) { test = true; }
    } else if(action.equals("connect")) {
        if(nodeid2 != "" && connectionType > -1) { test = true; }
    } else if(action.equals("disconnect")) {
        if(nodeid2 != "" && connectionType > -1) { test = true; }
    } else if(action.equals("sendmessage")) {
        if(nodeid2 != "" && connectionType > -1 && sendType != -1 * INT_MAX && bytes != -1 * INT_MAX) { test = true; }
    } else if(action.equals("deactivate")) {
        if(nodeid2 == "" && connectionType == -1) { test = true; }
    }
 
    return test;   
  }
  
  //executes the current command
  //returns true if successfull, false the id didn't exist or command not valid
  public boolean execute() {
     if(!isValid) { return false; }
     
     if(action.equals("activate")) {
        return addNode(nodeid1);
    } else if(action.equals("connect")) {
        return connectNodes(nodeid1, nodeid2, connectionType);
    } else if(action.equals("disconnect")) {
        return disconnectNodes(nodeid1, nodeid2, connectionType);
    } else if(action.equals("sendmessage")) {
        return sendMessage(nodeid1, nodeid2, connectionType, bytes);
    } else if(action.equals("deactivate")) {
        return deactivateNode(nodeid1);
    }
     
     return false;
  }
  
  //adds a node to the graph
  //returns true if added successfully, false if a node with this id already exists
  public boolean addNode(String id) {
    if(graph.getNode(id) != null) { return false; }

    graph.add(new Node(id)); 
    return true;
  }

  //puts a link between nodes
  //returns true if successful, false if either one of the ids doesn't exist
  public boolean connectNodes(String id1, String id2, int connectionType) {
     Node node1 = graph.getNode(id1);
     Node node2 = graph.getNode(id2);
     
     if(node1 == null || node2 == null) { return false; }
   
     return node1.connect(node2, connectionType);
  }

  //deletes the link between nodes
  //returns true if successful, false if either one of the ids doesn't exist
  public boolean disconnectNodes(String id1, String id2, int connectionType) {
     Node node1 = graph.getNode(id1);
     Node node2 = graph.getNode(id2);
   
     if(node1 == null || node2 == null) { return false; }
   
     return node1.disconnect(node2, connectionType);
  }

  //sends a message between the two nodes
  //returns true if successful, false if either one of the ids doesn't exist
  //also returns false if there is no link between the nodes
  public boolean sendMessage(String id1, String id2, int connectionType, long bytes) {
    Node node1 = graph.getNode(id1);
    Node node2 = graph.getNode(id2);
   
    if(node1 == null || node2 == null) { return false; }
    
    cmdrunner.displayAdd(this);
  
    return node1.sendMessage(node2, connectionType, bytes);
  }

  //deletes the node and all of its links
  //returns false if invalid idor if node was already inactive
  public boolean deactivateNode(String id) {
     Node node1 = graph.getNode(id);
     if(node1 == null) { return false; }
     return node1.deactivate();
  }  
  
} //end Command class


class CommandRunner {

 //dont edit these
 int executeTime = 0;
 int frameCounter = 0;
 ArrayList queue = new ArrayList(0);
 ArrayList displayqueue = new ArrayList(0);
 
 BufferedReader gzipReader = null;
 Command tempCommand = null;
 Command breakCommand = null;
 String hostcatcher = "";
 int requestType = 1000;
 int resultType = 1001;
 int firstTimestamp = 0;
 boolean isBreaked = false;
 boolean eof = false;
 boolean fileproblem = false;
 
 Command lastExecuted;
 
CommandRunner(String filename) {

   initLog(filename);
   
   tempCommand = getNextCommand();
   
   if(tempCommand != null) {
      hostcatcher = tempCommand.nodeid1;
      executeTime =  tempCommand.timestamp;
      firstTimestamp = executeTime;
   }
 } 
 
//Returns the speed of the simulation, which is based off of the slider value
//We need this because the slider is linear, but we need to have a controllable range of ~0.01 to 100,000
//I determined these functions with best fit lines using a few data points
public float getSimulationSpeed() {
  int sliderVal = speedslider.getValue();
  float speed;
  
  if(sliderVal > 47 && sliderVal < 52) { speed = 1.0f; }
  else if(sliderVal < 50) { speed = sliderVal / 50.0f; }
  else { speed = 6e-6f*pow(1.25f, sliderVal+3); } 
  
  //truncate excessive precision
  if(speed < 10) { speed = Math.round(speed * (1 / .01f)) / (1 / .01f); }
  else { speed = Math.round(speed); }
  
  return speed;
  
}
 
//adds a command to the display
public void displayAdd(Command cmd) {
  int maxSize = 13;
  
  //if its a result, add it to the queue and check maxSize
  if(isSearchResultConnection(cmd.connectionType) && displayqueue.size() > 0) {
     displayqueue.add(cmd);
     if(displayqueue.size() > maxSize) {
        displayqueue.remove(1); 
     } 
  }
  
  //if its a request, reset the queue if its a new one, else ignore it
  if(isSearchConnection(cmd.connectionType)) {
      if(displayqueue.size() == 0) { 
        displayqueue.add(cmd); 
        if(breakbox.isChecked()) { isBreaked = true; breakCommand = cmd;
           step._visible = true;
           resume._visible = true;
        }
        return;
      }
     //check to see if its an old or new request 
      Command temp = (Command)displayqueue.get(0);
      if(!temp.comment.equals(cmd.comment)) {
         displayqueue.clear();
         //we need to break and cancel all further cmd processing here.
         if(breakbox.isChecked()) { isBreaked = true;  breakCommand = cmd;
           step._visible = true;
           resume._visible = true;
        }
         displayqueue.add(cmd);
      }
  }
  
}

//draws all the commands on the display
public void displayDraw() {
  
  Command cmd;
  String message = "";
  String resultMessage;
  int textColor;
  
  for(int i = 0; i < displayqueue.size(); i++) {
      cmd = (Command)displayqueue.get(i);
      if(isSearchConnection(cmd.connectionType)) {
        message = cmd.nodeid1 + " - \"" + cmd.comment + "\"";
        textColor = orange;
      }
      else {      
        message = "   " + cmd.nodeid1 + " - " + cmd.comment + " (" + bytesToString(cmd.bytes) + ")";
        textColor = blue;
      }
      drawMessage(message, width-250, i*20 + 150, textColor, Franklin);
     }
  
}

//this should be called by draw at every tick
public void execute() {
  
  int seconds = 0;
  int totalFrames = 0;
  float dataPerSecond = getSimulationSpeed();
  
  //we can draw at least one second of data per frame
  if(dataPerSecond >= framerate) {
     seconds = (int)(dataPerSecond/framerate);
  } 
  //otherwise we must skip frames to play data slow enough
  else {
     seconds = 1;
     totalFrames = (int)(framerate/dataPerSecond);
     if(frameCounter < totalFrames - 1) {
       frameCounter++;
       return; 
     } else {
       frameCounter = 0;
     }
  }
  
  dumbIntoQueue(seconds);  
  runQueueAndClear();
  
}

//seeks to a timestamp
public void seek(int timestamp) {
   
   while(tempCommand != null && tempCommand.timestamp < timestamp) {
     tempCommand = getNextCommand();
     if(tempCommand != null) { queue.add(tempCommand); }
  }
   
  executeTime = tempCommand.timestamp;
  runQueueAndClear();
  isBreaked = true;
  breakCommand = tempCommand;
  step._visible = true;
  resume._visible = true;
}

//Runs exactly one command
public void step() {
    
  if(queue.size() > 0) {
      breakCommand = (Command)queue.get(0);
      queue.remove(0);
      breakCommand.execute();
      lastExecuted = breakCommand;
  } else {
      breakCommand = getNextCommand();
      if(breakCommand != null) { breakCommand.execute();  lastExecuted = breakCommand; }
  }
}

//dumps seconds worth of data into the queue (relative to executeTime)
//if executeTime is -1 then relative to first command's timestamp
public void dumbIntoQueue(int seconds) {
  
    seconds *= 1000; //turn seconds into milliseconds for reading the log
    
    if(tempCommand != null && tempCommand.timestamp < executeTime + seconds) {
      if(tempCommand.nodeid1 != hostcatcher) {
        queue.add(tempCommand);
      }
    }
    else if(tempCommand != null) {
      executeTime += seconds;
      return;
    }
    
    while(true) {
      tempCommand = getNextCommand();
      if(tempCommand != null && tempCommand.timestamp < executeTime + seconds) {
        queue.add(tempCommand);
      }
      else
        break;
    }
    
    executeTime += seconds;
    
}

//executes all commands in the queue and clears it
public void runQueueAndClear() { 
  while(queue.size() > 0) {
      breakCommand = (Command)queue.get(0);
      queue.remove(0);
      if(!(breakCommand.nodeid1.equals(hostcatcher) || (breakCommand.nodeid2 != "" && breakCommand.nodeid2.equals(hostcatcher))))
        breakCommand.execute();
      lastExecuted = breakCommand;
      if(isBreaked) { return; }
  }
}

//returns the next valid command that is in the log file.
public Command getNextCommand() {
  Command cmd = null;
  String line;
  
 if(gzipReader != null) {
   try {
         if(!gzipReader.ready()) { 
           gzipReader.close(); 
           eof = true;
           return null;
          }
         while(cmd == null || !cmd.isValid || (cmd.nodeid1.equals(hostcatcher) || (cmd.nodeid2 != "" && cmd.nodeid2.equals(hostcatcher)))) {
           line = gzipReader.readLine();
           if(line != null) { cmd = new Command(line); } 
           else {
              eof = true;
              return null; 
           }
         }
     } catch (FileNotFoundException fnfe) {
         fileproblem = true;
      } catch (IOException ioe) {
         fileproblem = true;
      } 
 }
  
  return cmd;
 
}

//initializes the log file input stream
public void initLog(String filename) {
  InputStream filestream = openStream(filename);
  
   // use BufferedReader to get one line at a time
      try {
         gzipReader = new BufferedReader(
               new InputStreamReader(
                new GZIPInputStream(filestream)));
   
     } catch (NullPointerException npe) {
        fileproblem = true;
     } catch (FileNotFoundException fnfe) {
         fileproblem = true;
      } catch (IOException ioe) {
         fileproblem = true;
      }
    
}

}// end CommandRunner class


// The graph that is displayed by the graph viewer
class Graph {
 
 Hashtable nodes = new Hashtable(400);
 Node source = null;
 boolean radialDirty = true;
 long maxLinkBytes;
 long minLinkBytes;
 long maxNodeBytes;
 long minNodeBytes;

 //constructor
 Graph() {

 } 
 
 //adds a node to the hashtable
 public void add(Node anode) {
   nodes.put(anode.id, anode); 
   
   //how to determine original center node?
   if(source == null) {
     source = anode;
   }
 }
 
 //returns a Node object that has the specified id, or null if not found
 public Node getNode(String id) {
  return (Node)nodes.get(id);
 }
 
 //returns the radius of the radial circles, based on the position of the zoomslider
 public float getRadialRadius() {
    int min = 10, max = 275;
    
    float frac = zoomslider.getValue() / 100.0f;
    
    return frac*(max-min) + min;
 }
 
 //draws all of the nodes
 public void drawNodes() {
   Enumeration nodeenum;
   Enumeration linkenum;
   long bytes;
   Node node;
   Link link;
   
   strokeJoin(ROUND); strokeCap(ROUND);
   fill(white);
   textAlign(LEFT);
   
   //compute maximum and minimum link bytes
   maxLinkBytes = -1;
   minLinkBytes = -1;
   maxNodeBytes = -1;
   minNodeBytes = -1;
   for(nodeenum = nodes.elements() ; nodeenum.hasMoreElements() ;) {
       node = (Node)nodeenum.nextElement(); 
       bytes = node.uplink + node.downlink;
       if(bytes > maxNodeBytes || maxNodeBytes == -1)
         maxNodeBytes = bytes;
       if(bytes < minNodeBytes || minNodeBytes == -1)
         minNodeBytes = bytes;
       for(linkenum = node.links.elements() ; linkenum.hasMoreElements() ;) {
         link = (Link)linkenum.nextElement();
         bytes = link.bytes;
         if(isSearchConnection(link.connectionType)) {
           if(bytes > maxLinkBytes || maxLinkBytes == -1)
             maxLinkBytes = bytes;
           if(bytes < minLinkBytes || minLinkBytes == -1)
             minLinkBytes = bytes;
         }
       }
     }
   
   //draw the nodes
   for(nodeenum = nodes.elements() ; nodeenum.hasMoreElements() ;) {
         node = ((Node)nodeenum.nextElement());
         node.drawMe();
     }
 }
 
 //draws all of the links
 public void drawLinks() {
     Enumeration nodeenum;
     Enumeration linkenum;
     Node node;
     Link link;
     
     if(linksbox.isChecked() || transitionMode || graph.nodes.size() > 100) { noSmooth(); }
     else { smooth(); }
     strokeJoin(ROUND); strokeCap(ROUND);
     strokeWeight(1); 
     
     for(nodeenum = nodes.elements() ; nodeenum.hasMoreElements() ;) {
       node = (Node)nodeenum.nextElement(); 
       for(linkenum = node.links.elements() ; linkenum.hasMoreElements() ;) {
         link = (Link)linkenum.nextElement();
         link.drawMe();
       }
     }
 }
 
 //returns the closest node to the (x,y) coordinate passed in
 //or null if no node within tolerance
 public Node getClosestNode(int x, int y) {
  Enumeration nodeenum;
  Node n, closest = null;
  float dist, min = FLT_MAX;
  
  //return null if mouse is in the infobox
  if(x + xtranslate > 735) { return null; }
  
  //fidn the closest node
  for(nodeenum = nodes.elements() ; nodeenum.hasMoreElements() ;) {
          n = (Node)nodeenum.nextElement();
          dist = distance(n.getX(), n.getY(), x, y);
          
          if(dist < min) {
            min = dist;
            closest = n;
          }
      }
      
  //if mouse is outside the bounds of closest return null
  if(closest != null && distance(closest.getX(), closest.getY(), x, y) > closest.getDiameter()/2.0f)
    return null;
  
  return closest;
 } 
 
 //returns the node matching the string in the find text box.
 public Node getClosestNode(String nodeid) {
  Enumeration nodeenum;
  Node testNode = null;
  
  for(nodeenum = nodes.elements() ; nodeenum.hasMoreElements() ;) {
          testNode = (Node)nodeenum.nextElement();
          if(testNode.id.indexOf(nodeid) == 0) { return testNode; }
      }
  
  return null;
 } 
 
 //resets maxBytes, bytes for links, and uplink and downlink for all nodes
 public void resetStats() { 
   Node node;
   Link link;
   Enumeration nodeenum;
   Enumeration linkenum;
   
   maxLinkBytes = 0;
   maxNodeBytes = 0;
   cmdrunner.displayqueue.clear();
     
     for(nodeenum = nodes.elements() ; nodeenum.hasMoreElements() ;) {
       node = (Node)nodeenum.nextElement(); 
       node.uplink = 0;
       node.downlink = 0;
       for(linkenum = node.links.elements() ; linkenum.hasMoreElements() ;) {
         link = (Link)linkenum.nextElement();
         link.bytes = 0;
       }
     }
 }

 //Changes positions of nodes to match correct radial layout with source at the center
 public void radialLayout() {
     ArrayList queue = new ArrayList();
     Node iterator, parent, child;
     Link link;
     Enumeration nodeenum;
     Enumeration linkenum;
   
   //initialize
   if(source != null) {
    for(nodeenum = nodes.elements() ; nodeenum.hasMoreElements() ;) {
           iterator = (Node)nodeenum.nextElement();
           iterator.oldd = iterator.d;
           iterator.d = INT_MAX;
           iterator.parent = null;
       }
    source.d = 0;

    //perform BFS
    queue.add(source);
    while(queue.size() > 0) {
       parent = (Node)queue.remove(0);
       for(linkenum = parent.links.elements() ; linkenum.hasMoreElements() ;) {
           link = (Link)linkenum.nextElement();
           if(isSearchConnection(link.connectionType)) {
              child = link.otherNode(parent);
              if(child.d == INT_MAX) {
               child.d = parent.d + 1;
               child.parent = parent;
               queue.add(child);
              } 
           }
       }
    }
   
    //palce bfs nodes
    source.getNumberOfLeaves();
    source.place(0, TWO_PI);
    //place all nodes with no parent out of the screen
    for(nodeenum = nodes.elements() ; nodeenum.hasMoreElements() ;) {
           iterator = (Node)nodeenum.nextElement();
           if(iterator.parent == null && iterator != graph.source) { iterator.d = INT_MAX; }
       }
   //lets get outta here
   }
 } //end radialLayout()
 
 //puts the graphview in transition mode, with node being the new center.
 public void beginTransition(Node node) {
  if(node != null && !transitionMode && (source == null || node != source)) {
    source = node; 
    transitionMode = true;
    //computer the new positions
    radialDirty = true;
   } 
 }

} //end Graph class


//a Link object connects two nodes
class Link {
   Node node1;
   Node node2;
   int connectionType;
   
   long bytes = 0;
   long lastDrawFrame = -1;
   
   //constructor
   Link(Node one, Node two, int connectType) {
      node1 = one;
      node2 = two;
      connectionType = connectType;
   }
   
   //This will be called by one of the nodes connected by the link
   //it will return the OTHER node, since it could be node1 or node2
   public Node otherNode(Node node) {
      if(node == node1) { return node2; }
      if(node == node2) { return node1; }
      return null;
   }
   
   //draws the line between node1 and node2
   public void drawMe() {
     
     if(isDisplayLink() && aboveThreshold() && node1.active && node2.active && lastDrawFrame < frameCount) {
      if(isTreeLink() || linksbox.isChecked()) {
        //draw the link
        stroke(getLinkColor());
        
        beginShape(LINE_STRIP);  vertex(node1.getX(), node1.getY()); vertex(node2.getX(), node2.getY());  endShape();
      
        //show label
        float xlabel = node2.getX() - (node2.getX() - node1.getX()) / 2;
        float ylabel = node2.getY() - (node2.getY() - node1.getY()) / 2;
      
        fill(black);
        //text("B:" + bytes, xlabel, ylabel);
        
        lastDrawFrame = frameCount;
      }
     }
   }
 
 //returns true if this link is a search or result connection 
 public boolean isDisplayLink() {
   return (isSearchConnection(connectionType) || isSearchResultConnection(connectionType));
 }
 
 //returns true if this link is in the BFS tree
 //NOTE: also returns true for some other important links
 public boolean isTreeLink() {
   if(node1.parent == node2 || node2.parent == node1)
     return true;
   if(isSearchResultConnection(connectionType))
     return true;
   if(node1.isPartOfBreakMessage() && node2.isPartOfBreakMessage())
     return true;
   return false;
 }
 
 //returns true if this link is above the link threshold, based on the bytes sent across it.
 public boolean aboveThreshold() {
   float frac = 0;
   
   if(threshslider.getValue() == 100) { return true; }
   if(graph.maxLinkBytes == 0) { return true; }
   if(isSearchResultConnection(connectionType)) { return true; }
   if(node1.isPartOfBreakMessage() && node2.isPartOfBreakMessage()) { return true; }
   if(threshslider.getValue() == 0) { return false; }
   
   frac = (1.0f*this.bytes) / graph.maxLinkBytes;
   
   return frac >=  1 - (threshslider.getValue() / 100.0f);
 }
  
 //returns the link color, based upon this link's bytes compared to graph's maxLinkBytes
 //fades from green -> yellow -> orange -> red 
 public int getLinkColor() {
      float frac = 0;
      int maxColor = 240, minColor = 10;
      int linkcolor;
      
      if(isSearchResultConnection(connectionType)) { return blue; }
          
      //get the fraction from minLinkBytes to maxLinkBytes
      if(graph.maxLinkBytes > 0 && (graph.maxLinkBytes-graph.minLinkBytes) > 0) {
        frac = (1.0f*this.bytes-graph.minLinkBytes) / (graph.maxLinkBytes-graph.minLinkBytes);
      }
      
      //determine the color
      if(frac < 0.5f) {
        linkcolor = color((frac / 0.5f) * (maxColor-minColor) + minColor, maxColor, minColor);
      } else {
        linkcolor = color(maxColor, (1.0f - ((frac-.5f)/.5f)) * (maxColor-minColor) + minColor, minColor); 
      }
     
     return linkcolor;
 }
 
} //end Link class


// A node object represents a point on the graph.
// This will be the individial peer.
class Node {
  
  //General variables
  String id;
  boolean active = true;
  long uplink = 0;
  long downlink = 0;
  
  //radial varaibles
  Node parent = null;
  int numLeaves;
  int d = INT_MAX, oldd = INT_MAX;
  float angle, oldangle;
  
  Hashtable links = new Hashtable(50); 
   
   //constructor
   Node(String myid) {
    id = myid;
   }

   //instatiates a link object and adds it to both node's link arrays
   //returns true is successful, false if anothernode is null or inactive
   public boolean connect(Node anothernode, int connectionType) {
     
     if(anothernode == null || anothernode == this || !anothernode.active) { return false;  }
     
     //make sure that we aren't already connected
      boolean alreadyconnected = false;
      if(links.get(anothernode.id + "-" + connectionType) != null) { alreadyconnected = true; }

      //create a link object and add it to both nodes link arrays
      if(!alreadyconnected) {
       Link alink = new Link(this, anothernode, connectionType);
       this.links.put(anothernode.id + "-" + connectionType, alink);
       anothernode.links.put(this.id + "-" + connectionType, alink);
       graph.radialDirty = true;
      }
      return true;
   }
   
   
   //sends a message to the destination via the link that connects them
   //returns true if successful
   public boolean sendMessage(Node destination, int connectionType, long bytes) {
      //there's a problem, don't do this
      if(destination == null || !destination.active || destination == this) { return false; }
      
      //we want to ignore all uninteresting stuff
      if(!isSearchConnection(connectionType) && !isSearchResultConnection(connectionType)) { return false; } 
      if(id.equals(cmdrunner.hostcatcher) || destination.id.equals(cmdrunner.hostcatcher)) { return false; }
     
      Link thelink = (Link)links.get(destination.id + "-" + connectionType);
      Node otherNode = thelink.otherNode(this);///////null?
      
      if(thelink != null && bytes > 0) {
     
        //update link bandwidth
        thelink.bytes += bytes;
        
        //update my uplink bandwidth
        uplink += bytes;
        
        //update their downlink bandwidth
        otherNode.downlink += bytes;
        
        return true;
      }
      
      return false;
   }
   
   //deletes the link from this node to anothernode
   //returns true is successful, false if anothernode is null
   public boolean disconnect(Node anothernode, int connectionType) {
     
     if(anothernode == null || anothernode == this) { return false; }
     
     this.links.remove(anothernode.id + "-" + connectionType);
     anothernode.links.remove(this.id + "-" + connectionType);
     
     graph.radialDirty = true;
     
     return true;
   }
   
   //sets active as false and removes all links joining this node
   //returns false is node was already not active.
   public boolean deactivate() {
     Enumeration linkenum;
     
      if(!active) { return false ; }
      
      //remove the link from the other node's link array
      for(linkenum = links.keys() ; linkenum.hasMoreElements() ;) {
         String linkname = ((String)linkenum.nextElement());
         String nodeid = linkname.substring(0, linkname.indexOf('-'));
         String connectionType = linkname.substring(linkname.indexOf('-') + 1);
         graph.getNode(nodeid).links.remove(this.id + "-" + connectionType);
      }
      
      //delete all links in this node
      links.clear();
      
      active = false;
      graph.radialDirty = true;
      return true;
   }
   
   //returns the correct x position of the node with 0 being the center of the graph view.
   public float getX() {
       if(!transitionMode)
         return graph.getRadialRadius()*d*cos(angle) + width/2;
       else {
         float oldpos = graph.getRadialRadius()*oldd*cos(oldangle) + width/2;
         float newpos = graph.getRadialRadius()*d*cos(angle) + width/2;
         float total = newpos - oldpos;
         return oldpos + getTransitionFraction()*total; 
       }
   }
   
   //returns the correct y position of the node with 0 being the center of the graph view.
   public float getY() {
       if(!transitionMode)
         return graph.getRadialRadius()*d*sin(angle) + height/2;
       else {
         float oldpos = graph.getRadialRadius()*oldd*sin(oldangle) + height/2;
         float newpos = graph.getRadialRadius()*d*sin(angle) + height/2;
         float total = newpos - oldpos;
         return oldpos + getTransitionFraction()*total; 
       }
   }
   
   //draws the node primative at its (x,y) coordinate
   public void drawMe() {
     if(active) {
  
        stroke(lightblue);
        
        //see if this node appears in the info box on the right, color appropriately
        Command cmd;
        for(int i = cmdrunner.displayqueue.size() - 1; i >= 0; i--) {
           cmd = (Command)cmdrunner.displayqueue.get(i);
           if(isSearchResultConnection(cmd.connectionType) && cmd.nodeid1.equals(this.id)) {//isResult(cmd.sendType)
               stroke(blue);
           }
           if(isSearchConnection(cmd.connectionType) && cmd.nodeid1.equals(this.id)) {//isRequest(cmd.sendType)
               stroke(orange);
           }
        }
        
        //determine radius 
        strokeWeight(getDiameter());
        
        if(graph.nodes.size() > 100) { noSmooth(); }
        else { smooth(); }  
        beginShape(POINTS);  vertex(getX(), getY()); endShape();
        
        if(ipbox.isChecked() || isPartOfBreakMessage()) { drawMessage(id, getX()+10, getY()-10, white, Franklin); }
     }
   }
   
 //returns true if we are breaked and this node is part of the break message
 public boolean isPartOfBreakMessage() {
    if(cmdrunner.isBreaked && cmdrunner.breakCommand != null) { //////////////??
       if(id.equals(cmdrunner.breakCommand.nodeid1))
         return true;
       if(id.equals(cmdrunner.breakCommand.nodeid2))
         return true;
    }
    return false;
 }
  
 //returns the radius of this node based on graph.MaxNodeBytes 
 public float getDiameter() {
   float frac = 0;
   int min = 10, max = 40;
   
   if(graph.maxNodeBytes > 0 && (graph.maxNodeBytes-graph.minNodeBytes) > 0) {
        frac = (1.0f*(uplink+downlink)-graph.minNodeBytes) / (graph.maxNodeBytes-graph.minNodeBytes);
   }

   return frac*(max-min) + min;
 } 
 
 //counts the number of children that this node has in the BFS tree
 public int getNumberOfChildren() {
   Enumeration linkenum;
   int numChildren = 0;
   Link link;
   Node child;
   
   //count children
  for(linkenum = links.elements() ; linkenum.hasMoreElements() ;) {
         link = (Link)linkenum.nextElement();
         if(isSearchConnection(link.connectionType)) {
           child = link.otherNode(this);
           if(child.parent == this) { //it really is a child
             numChildren++;
           }
         }
    }
    
   return numChildren;
 }
   
 //recurses on all children to find number of leaves
 //note: the numLeaves variable is in the score of Node and gets updated.
 public int getNumberOfLeaves() {
  Enumeration linkenum;
  int leaves = 0;
  Link link;
  Node child;  
  
  numLeaves = 0;
 
   if(getNumberOfChildren() == 0) { numLeaves = 0; return 1;} //we are  a leaf
  
   for(linkenum = links.elements() ; linkenum.hasMoreElements() ;) {
         link = (Link)linkenum.nextElement();
         if(isSearchConnection(link.connectionType)) {
           child = link.otherNode(this);
           if(child.parent == this) { //it really is a child
             numLeaves += child.getNumberOfLeaves();
           }
         }
      }
    
    return numLeaves;
 
 }
 
 //places this node on the correct position on its circle, the recurses to child nodes
 public void place(float start, float end) {
     Enumeration linkenum;
     float total = end - start;
     int childleaves;
     float fraction;
     Link link;
     Node child;
     
     //compute angle for this node
     oldangle = angle;
     angle = start + total/2.0f;
     
      //for each child
      for(linkenum = links.elements() ; linkenum.hasMoreElements() ;) {
         link = (Link)linkenum.nextElement();
         
         if(isSearchConnection(link.connectionType)) {
           child = link.otherNode(this);
           if(child.parent == this) { //it really is a child
             childleaves = child.numLeaves;
             if(childleaves == 0) { childleaves++; }
             fraction = (float)childleaves / this.numLeaves;
             end = start + fraction * total;
             
             child.place(start, end);
             start = end;
           }
         }
      }
 }
  
} //end Node class
static public void main(String args[]) {   PApplet.main(new String[] { "graphviewer" });}}