
// Graph viewer
// Team SPEW
// Direct question to eric.withrow@gmail.com

import mkv.MyGUI.*;

// Editable Variables    
String defaultONAfile = "default.ona.gz";
int framesPerSecond = 20;                     // This is a suggestion only, actual framerate is calculated in variable "framerate"
int totalTransitionFrames = 20;               // The number of frames it takes for the nodes to transition
int initialZoom = 30;                         // The initial zoom (in percentage), min and max values are determined in graph.getRadialRadius()
int xtranslate = -125, ytranslate = 0;        // The initial placement of the center of the graph

// Non-editable variables
Graph graph = new Graph();
CommandRunner cmdrunner;
int INT_MAX = 99999;
float FLT_MAX = 99999;
boolean transitionMode = false;
int transitionFrame = 0;
String ONAfile = getONAfile();

// Colors & Fonts
color black = color(0, 0, 0);
color white = color(250, 250, 250);
color red = color(200, 0, 0);
color green = color(0, 200, 0, 190);
color blue = color(0, 0, 200, 200);
color gray = color(50, 50, 50);
color orange = color(240, 150, 0, 200);
color lightblue = color(66, 114, 204, 200);
color lightgray = color(226,226,226);
color bordergray = color(198,198,198);
PFont Verdana, Franklin;

//GUI variables
MyGUI gui;
MyGUIButton left, right, up, down, center, reset, go, step, resume, advancedmenu;
MyGUICheckBox linksbox, ipbox, pausebox, breakbox;
MyGUIPinSlider zoomslider, speedslider, threshslider;
MyGUITextInput findnode;
MyGUILabel zoomlabel, threshlabel, threshvallabel, speedlabel, speedvallabel;

//Advanced GUI variables
boolean advancedMode = false;
MyGUILabel  a_timestamplabel, a_requestlabel, a_resultlabel, a_hostcatcherlabel;
MyGUICheckBox a_circlesbox;
MyGUITextInput a_timestampinput, a_requestinput, a_resultinput, a_hostcatcherinput;
MyGUIButton a_save, a_timestampgo;

////////////////// Magic Number Methods ////////////////////
  //  1000 is SearchConnection
  //  1001 is SearchResultConnection
  //  1002 is IndexConnection
  // 10000 is HostcatcherRegisterConnection
  // 10001 is HostcatcherQueryConnection
  // generally we've only been interested in SearchConnection.

boolean isSearchConnection(int ct) {
  return (ct == cmdrunner.requestType);
}

boolean isSearchResultConnection(int ct) {
  return (ct == cmdrunner.resultType);
}

///////////////////// BEGIN PROGRAM /////////////////////
//this stuff happens at applet load time
void setup() { 
  size(1000, 680);                  // window size
  framerate(framesPerSecond);       // initialize window refresh rate
  background(black);
  Verdana = loadFont("Verdana-12.vlw");
  Franklin = loadFont("FranklinGothic-Book-11.vlw");
  cmdrunner = new CommandRunner(ONAfile);
  initGUI();
}

//the stuff in here happens at every tick
void draw() {
  
    //perform the transition, if necessary
    if(transitionMode) {
        transitionFrame++;
        if(transitionFrame > totalTransitionFrames) {
          transitionFrame = 0;
          transitionMode = false;
        }
    }
  
    //execute commands for this tick
    if(!transitionMode && !pausebox.isChecked() && !cmdrunner.isBreaked)
        cmdrunner.execute();
        
    //recompute BFS tree and place nodes
    if(graph.radialDirty) {
       graph.radialLayout();
       graph.radialDirty = false;
    }
    
    //If the GUI is being pressed, execute the right action
    guiPressAction();
    
    //DRAWING STUFF BELOW THIS POINT
    background(black);
    if(a_circlesbox.isChecked()) { drawCircles(15); }
    
    if(cmdrunner.fileproblem && !cmdrunner.eof) { drawMessage("Please make sure " + ONAfile + " is in the data directory and is a valid ONA file.", 10, 22, white, Franklin); }
    
    //translate to correct coordinates and draw links & nodes
    pushMatrix();
    translate(xtranslate, ytranslate);
    graph.drawLinks();
    graph.drawNodes();
    popMatrix();
      
    //draw the stuff related to the GUI on top of the canvas
    placeWidgets();
    drawInfoBox();
 }
 

//draws the info box on the right
void drawInfoBox() {
  noSmooth();
  stroke(bordergray);
  strokeWeight(2);
  rectMode(CORNER);
  
  //update labels
  String simSpeed = cmdrunner.getSimulationSpeed() + "x";
  if(pausebox.isChecked() && (frameCount/framesPerSecond) % 2 == 0) { simSpeed = "PAUSED"; }
  if(cmdrunner.isBreaked && (frameCount/framesPerSecond) % 2 == 0) { simSpeed = "BREAK"; }
  speedvallabel._text = "Sim Speed = " + simSpeed;
  threshvallabel._text = threshslider.getValue() + "%";
  
  //draw the break box
  fill(lightgray);
  rect(0,height-25,width,25);
  if(cmdrunner.isBreaked && cmdrunner.breakCommand != null) {
    drawMessage("BREAK ON: ", 10, height-8, black, Verdana);
    drawMessage(cmdrunner.breakCommand.originaltext, 80, height-8, black, Franklin);
  } else if(!cmdrunner.eof && cmdrunner.lastExecuted != null) {
      drawMessage("Executed Timestamp: " + cmdrunner.lastExecuted.timestamp + " @ " + fileName(ONAfile), 10, height-8, black, Verdana);
  } else if(cmdrunner.eof) {
    drawMessage("Reached end of " + fileName(ONAfile), 10, height-8, black, Verdana);
  }
  
  //draw the gray rectangle on right over the graph
  fill(lightgray);
  rect(width-265,0,265,height);
  
  if(!advancedMode) {
    if(graph.source != null) {
      drawMessage("Center Node: " + graph.source.id, width-250, 20, black, Verdana);
      drawMessage("   Downlink Transferred: " + bytesToString(graph.source.downlink), width-250, 40, black, Verdana);
      drawMessage("   Uplink Transferred: " + bytesToString(graph.source.uplink), width-250, 60, black, Verdana);
      drawMessage("   Total Transferred: " + bytesToString(graph.source.uplink + graph.source.downlink), width-250, 80, black, Verdana);
      drawMessage("   Number of Neighbors: " + graph.source.getNumberOfChildren(), width-250, 100, black, Verdana);
    }
  
    cmdrunner.displayDraw();  
  } else { //draw the Legend
      smooth();
      drawMessage("========Legend=======", width-240, height-250, black, Verdana);
      stroke(lightblue);
      strokeWeight(10);
      drawMessage("Data Transferred: " + bytesToString(graph.minNodeBytes), width-210, height-216, black, Verdana);
      smooth();
      beginShape(POINTS);  vertex(width-235, height-220); endShape();
      strokeWeight(40);
      drawMessage("Data Transferred: " + bytesToString(graph.maxNodeBytes), width-210, height-171, black, Verdana);
      smooth();
      beginShape(POINTS);  vertex(width-235, height-175); endShape();
      strokeWeight(1);
      stroke(color(10,240,10));
      drawMessage("Data Transferred: " + bytesToString(graph.minLinkBytes), width-240, height-110, black, Verdana);
      smooth();
      beginShape(LINE_STRIP);  vertex(width-250, height-130); vertex(width-140, height-130);  endShape();
      stroke(color(240,10,10));
      drawMessage("Data Transferred: " + bytesToString(graph.maxLinkBytes), width-240, height-60, black, Verdana);
      smooth();
      beginShape(LINE_STRIP);  vertex(width-250, height-80); vertex(width-140, height-80);  endShape();
  }
  
}

//draws the circles for radial mode
void drawCircles(int number) {
  noFill(); noSmooth(); strokeWeight(1); stroke(gray);
  pushMatrix();
  translate(xtranslate, ytranslate);
  for(int i = number; i >= 0; i--) {
    arc(width/2, height/2, graph.getRadialRadius()*2 * i, graph.getRadialRadius()*2 * i, 0, TWO_PI);
  }
  popMatrix();
}

//draws a message on the screen
void drawMessage(String message, float x, float y, color textColor, PFont font) {
  textFont(font);
  fill(textColor);
  textMode(MODEL);
  textAlign(LEFT);
  text(message, x, y); 
}

//listener for keyboard presses
void keyPressed() {

  if(key == CODED) { 
    if (keyCode == UP) {  ytranslate += 30; } 
    if (keyCode == DOWN) { ytranslate -= 30; }  
    if (keyCode == LEFT && !findnode.hasFocus()) { xtranslate += 30; } 
    if (keyCode == RIGHT && !findnode.hasFocus()) { xtranslate -= 30;  } 
  } else {
    if ((keyCode == RETURN || keyCode == ENTER) && findnode.hasFocus()) { goToNode(findnode.getValue()); }
    if ((keyCode == RETURN || keyCode == ENTER) && a_timestampinput.hasFocus()) { goToTimestamp(); }   
    if (key == '=' || key == '+') { zoomslider.setValue(zoomslider.getValue()+3);  } 
    if (key == '-' || key == '_') { zoomslider.setValue(zoomslider.getValue()-3); }
    if (key == 'l' || key == 'L') { linksbox.checked(!linksbox.isChecked());  } 
    if (key == 'z' || key == 'Z') { speedslider.setValue(speedslider.getValue()-1);  }
    if (key == 'x' || key == 'X') { speedslider.setValue(speedslider.getValue()+1);  }
    if (key == 'n' || key == 'N') { threshslider.setValue(threshslider.getValue()-5);  }
    if (key == 'm' || key == 'M') { threshslider.setValue(threshslider.getValue()+5);  }
    if (key == 'c' || key == 'C') { a_circlesbox.checked(!a_circlesbox.isChecked()); }
    if (key == 'o' || key == 'O') { ytranslate = 0; xtranslate = -125; zoomslider.setValue(initialZoom); }
    if (key == 'p' || key == 'P') { pausebox.checked(!pausebox.isChecked()); }
    if (key == 'i' || key == 'I') { ipbox.checked(!ipbox.isChecked()); }
    if (key == 'r' || key == 'R') { graph.resetStats(); }
  }
}

//If the buttons are being held we need to continually perform this action
void guiPressAction() {   
   if(left.isDragged()) { xtranslate += 30;}
   if(right.isDragged()) { xtranslate -= 30;  }
   if(up.isDragged()) { ytranslate += 30;  }
   if(down.isDragged()) { ytranslate -= 30;  }
}
 
// React if the button object was pressed
void actionPerformed(ActionEvent e) {
  if(e.getSource() == center) { zoomslider.setValue(initialZoom); ytranslate = 0; xtranslate = -125; }
  if(e.getSource() == reset) { graph.resetStats(); }
  if(e.getSource() == go) { goToNode(findnode.getValue()); }
  if(e.getSource() == step) { if(cmdrunner.isBreaked) { cmdrunner.step(); } }
  if(e.getSource() == advancedmenu) { switchAdvancedMode(); }
  if(e.getSource() == a_timestampgo) { goToTimestamp(); }
  if(e.getSource() == a_save) { 
    cmdrunner.requestType = stringToInt(a_requestinput.getValue());
    cmdrunner.resultType = stringToInt(a_resultinput.getValue());
    cmdrunner.hostcatcher = a_hostcatcherinput.getValue();
  }
  if(e.getSource() == resume) { 
    cmdrunner.isBreaked = false;
    step._visible = false;
    resume._visible = false;
   }
}

//When a person enters a string into the find node box and hits "Go", this happens
void goToNode(String findtext) {
   Node node = graph.getClosestNode(findtext);
   if(node == null) { 
     if(findnode.getValue().indexOf("Couldn't Find") < 0 && findnode.getValue().indexOf("Enter a node and hit") < 0)
       findnode.setValue("Couldn't Find \"" + findtext + "\""); 
   } else { 
     graph.beginTransition(node);
     findnode.setValue("Enter a node and hit \"go\"");
   }
}

//seeks to the timestamp in the timestamp input box.
//if it is invalid, displays an error.
void goToTimestamp() {
  String text = a_timestampinput.getValue();
  int timestamp = stringToInt(text);
  
  if(timestamp < cmdrunner.firstTimestamp) {
    a_timestampinput.setValue("Timestamp must be >= " + cmdrunner.firstTimestamp);
  } else {
    graph = new Graph();
    cmdrunner = new CommandRunner(ONAfile);
    cmdrunner.requestType = stringToInt(a_requestinput.getValue());
    cmdrunner.resultType = stringToInt(a_resultinput.getValue());
    cmdrunner.hostcatcher = a_hostcatcherinput.getValue();
    cmdrunner.seek(timestamp);
  }
  
}

//this happens when the mouse is pressed
void mousePressed() {
   
  //clear out the GoTo box if it has focus
  if(findnode.hasFocus()) 
    if(findnode.getValue().indexOf("Couldn't Find") == 0 || findnode.getValue().indexOf("Enter a node and hit") == 0)
      findnode.setValue("");

  Node node = graph.getClosestNode(mouseX-xtranslate, mouseY-ytranslate);
  graph.beginTransition(node);
}

////////////////// GUI Setup Methods ////////////////////
//Initializes all the gui widgets
void initGUI() {
  //Set up the GUI
  gui = new MyGUI(this, 50);
  gui.getStyle().setFont(Verdana, 12);
  gui.getStyle().tintColor(white); //mouseover EXCEPT for buttons
  gui.getStyle().buttonText = black; //color of LABEL text
  
  //Normal GUI elements
  breakbox = new MyGUICheckBox(this, width-240, height-225, "Break On Event");
  linksbox = new MyGUICheckBox(this, width-240, height-205, "Show All Links");
  ipbox = new MyGUICheckBox(this, width-115, height-205, "Show All IPs");
  left = new MyGUIButton(this, width-236, height-160, "<", 20, 16);
  center = new MyGUIButton(this, width-210, height-160, "-", 20, 16);
  right = new MyGUIButton(this, width-184, height-160, ">", 20, 16);
  up = new MyGUIButton(this, width-210, height-182, "/\\", 20, 16);
  down = new MyGUIButton(this, width-210, height-138, "\\/", 20, 16);
  zoomslider = new MyGUIPinSlider(this, width-85, height-170, 140, 15, 0, 100);
  zoomlabel = new MyGUILabel(this, "Zoom:", width-156, height-188);
  reset = new MyGUIButton(this, width-85, height-140, "Reset B/W Counters", 140, 16);
  threshslider = new MyGUIPinSlider(this, width-115, height-115, 126, 15, 0, 100);
  threshlabel = new MyGUILabel(this, "Threshold:", width-254, height-115);
  threshvallabel = new MyGUILabel(this, "100%", width-40, height-115);
  speedslider = new MyGUIPinSlider(this, width-105, height-90, 180, 15, 0, 100);
  speedlabel = new MyGUILabel(this, "Speed:", width-250, height-90);
  speedvallabel = new MyGUILabel(this, "Sim Speed = 1.0x", width-170, height-65);
  pausebox = new MyGUICheckBox(this, width-240, height-65, "Pause");
  findnode = new MyGUITextInput(this, width-150, height-40, 200, 20);
  go = new MyGUIButton(this, width-25, height-40, "Go", 30, 20);
  advancedmenu = new MyGUIButton(this, width-130, height-15, "Advanced Options & Legend", 240, 16);
  step = new MyGUIButton(this, width-390, height-13, "Step >", 50, 16);
  resume = new MyGUIButton(this, width-315, height-13, "Resume >>", 80, 16);
  
  gui.add(linksbox); gui.add(ipbox);
  gui.add(left); gui.add(center); gui.add(right); gui.add(up); gui.add(down);
  gui.add(zoomslider); gui.add(zoomlabel);
  gui.add(reset);
  gui.add(threshslider); gui.add(threshlabel); gui.add(threshvallabel);
  gui.add(speedslider); gui.add(speedlabel); gui.add(speedvallabel);
  gui.add(pausebox);
  gui.add(findnode); gui.add(go);
  gui.add(advancedmenu);
  gui.add(breakbox); gui.add(step); gui.add(resume);

  zoomslider.setValue(initialZoom);
  speedslider.setValue(50);
  threshslider.setValue(100);
  findnode.setValue("Enter a node and hit \"go\"");
  step._visible = false;
  resume._visible = false;
  
  //Advanced GUI elements
  a_circlesbox = new MyGUICheckBox(this, width-240, 30, "Show Circles");
  a_timestamplabel = new MyGUILabel(this, "Go to Timestamp:", width-250, 80);
  a_timestampinput = new MyGUITextInput(this, width-150, 100, 200, 20);
  a_timestampgo = new MyGUIButton(this, width-30, 100, "Go", 30, 20);
  a_requestlabel = new MyGUILabel(this, "Request Connection Types:", width-250, 150);
  a_requestinput = new MyGUITextInput(this, width-175, 170, 150, 20);
  a_resultlabel = new MyGUILabel(this, "Result Connection Types:", width-250, 190);
  a_resultinput = new MyGUITextInput(this, width-175, 210, 150, 20);
  a_hostcatcherlabel = new MyGUILabel(this, "Host Catcher:", width-250, 230);
  a_hostcatcherinput = new MyGUITextInput(this, width-175, 250, 150, 20);
  a_save = new MyGUIButton(this, width-220, 280, "Save", 60, 20);
  
  gui.add(a_circlesbox);
  gui.add(a_timestamplabel); gui.add(a_timestampinput); gui.add(a_timestampgo); 
  gui.add(a_requestlabel); gui.add(a_requestinput); gui.add(a_resultlabel); gui.add(a_resultinput); 
  gui.add(a_hostcatcherlabel); gui.add(a_hostcatcherinput); gui.add(a_save);
  
  a_circlesbox.checked(true);
  a_requestinput.setValue(new Integer(cmdrunner.requestType).toString());
  a_resultinput.setValue(new Integer(cmdrunner.resultType).toString());
  a_timestampinput.setValue(new Integer(cmdrunner.firstTimestamp).toString());
  a_hostcatcherinput.setValue(cmdrunner.hostcatcher);
  
  //set initial state
  placeWidgets();
}

//switched the infobar to the advanced menu screen and back.
void switchAdvancedMode() {
   advancedMode = !advancedMode;
   if(advancedMode) { advancedmenu._text = "Back to Main Menu"; } 
   else { advancedmenu._text = "Advanced Options & Legend"; } 
}

//Determines correct widget state
void placeWidgets() {
   breakbox._visible = !advancedMode;
   linksbox._visible = !advancedMode; ipbox._visible = !advancedMode;
   left._visible = !advancedMode; center._visible = !advancedMode;  right._visible = !advancedMode;  up._visible = !advancedMode;  down._visible = !advancedMode; 
   zoomslider._visible = !advancedMode; zoomlabel._visible = !advancedMode;
   reset._visible = !advancedMode;
   threshslider._visible = !advancedMode; threshlabel._visible = !advancedMode; threshvallabel._visible = !advancedMode;
   speedslider._visible = !advancedMode; speedlabel._visible = !advancedMode; speedvallabel._visible = !advancedMode;
   pausebox._visible = !advancedMode;
   findnode._visible = !advancedMode; go._visible = !advancedMode;
   reset._visible = !advancedMode;
   
  a_circlesbox._visible = advancedMode;
  a_timestamplabel._visible = advancedMode; a_timestampinput._visible = advancedMode; a_timestampgo._visible = advancedMode; 
  a_requestlabel._visible = advancedMode; a_requestinput._visible = advancedMode; a_resultlabel._visible = advancedMode; a_resultinput._visible = advancedMode; 
  a_hostcatcherlabel._visible = advancedMode; a_hostcatcherinput._visible = advancedMode; a_save._visible = advancedMode;
}

////////////////// Global Util Methods ////////////////////

//returns the percentage of the way through the transition
//from 0 to 1, right now uses slow-in/slow-out transition
float getTransitionFraction() {
   float stepFraction = (1.0*transitionFrame) / totalTransitionFrames;
   return atan((stepFraction) * 10 - 5) * 0.5 / atan(5) + 0.5; 
}

//returns the integer value of the string.
  int stringToInt(String str) {
    try {
      return (new Integer(str.trim())).intValue();
    } catch(NumberFormatException e) {
     return -1 * INT_MAX;
    }
  }
  
 //takes in a number of bytes and returns it in pretty form, for example 21.4 MB
 String bytesToString(long bytes) {
        float val;
	String answer;
	
	if(bytes >= pow(2, 30)) {
		val = round(bytes / pow(2, 30) * 100.0) / 100.0;
		answer = val + " GiB";
	} else if(bytes >= pow(2, 20)) {
		val = round(bytes / pow(2, 20) * 100.0) / 100.0;
		answer = val + " MiB";
        } else if(bytes >= pow(2, 10)) {
		val = round(bytes / pow(2, 10) * 100.0) / 100.0;
		answer = val + " KiB";
	} else
		answer = bytes + " B";
	
	return answer;
 }
 
//returns the filename without any path information
String fileName(String path) {
  File file = new File(path);
  return file.getName();
}
 
 //returns the distance between two points
 float distance(float x1, float y1, float x2, float y2) {
   return sqrt((x1-x2)*(x1-x2) + (y1-y2)*(y1-y2));
 } 

 
// Don't call this from inside setup.
// application will probably deadlock.
//
// We weren't sure if we should have a file chooser or if using Swing components within Processing (AWT)
// would cause problems.  For that reason, this file choosing code is all contained in this method.
// I don't even use imports - I fully qualify the class names.  So if this method is giving you troubles
// or you just don't need a file chooser, remove this method and the one call to it.  Done!
//  - Chris Sidi
String getONAfile() {
  try {
    javax.swing.UIManager.setLookAndFeel(javax.swing.UIManager.getSystemLookAndFeelClassName());
  } 
  catch (Exception e) 
  {}
  
  String rv = defaultONAfile;
  
  try {
    SecurityManager security = System.getSecurityManager();
    if (null != security) {
      security.checkRead(".");
    }
    
    javax.swing.filechooser.FileFilter ff = new javax.swing.filechooser.FileFilter()
    {
      public boolean accept(File f) {
          if (f.isDirectory()) {
              return true;
          }

          return f.getName().toLowerCase().endsWith(".ona.gz");
      }
      
      public String getDescription() {
        return "Compressed ONA files (*.ona.gz)";
      }
    };
    
    javax.swing.JFileChooser fc = new javax.swing.JFileChooser(new File("."));
    fc.setFileFilter(ff);
    while (javax.swing.JFileChooser.APPROVE_OPTION != fc.showOpenDialog(null)) { 
      //cancel was pressed here, just return the default
      return defaultONAfile;
    }
  
    rv = fc.getSelectedFile().toString();
  }
  catch (SecurityException ex)
  {
    System.out.println("INFO: Not allowed to launch file dialog.  Defaulting to : "+ defaultONAfile +" ("+ ex +")");  
  }
  
  return rv;
}
