/**
 * ODIN overlay management middleware toolkit
 * 
 * ODIN Group, College of Computing, Georgia Institute of Technology
 *
 * For more information:
 * http://www.cc.gatech.edu/~cooperb/odin/
 * odin.gatech.edu
 *
 * You are free to use, modify and redistribute this software. 
 * However, we make no warranty for this software, and by downloading it
 * you agree not to hold use liable for anything. Do not under any
 * circumstances use this software to share copyrighted content or do
 * anything illegal. This software was designed for academic research
 * purposes and not for illegal activities. We are not responsible for
 * anything illegal you do and by downloading this software you agree
 * not to break any laws. Enjoy!
 *
 * Brian F. Cooper
 * November 21, 2005
 */

#ifndef LOGGER_H
#define LOGGER_H

const int LoggerSilent=0;
const int LoggerUrgent=1;
const int LoggerInfo=2;
const int LoggerVis=3;
const int LoggerDebug=4;

const int DefaultLevel=LoggerInfo;

/**
 * Object for logging messages. Messages are printed to the console
 * or delivered to an appropriate GUI. 
 *
 * This class prints everything to stdout. Subclasses can override message() to display messages in different ways.
 */
class Logger
{
  public:
   /**
    * Constructor. 
    */
   Logger(int _messagelevel=DefaultLevel);
   
   /**
    * Issue a message. The message is only issued if the level<=messagelevel.
    */
   virtual void message(int level, const char * msg);   

   /**
    * Change the message level.
    */
   virtual void setMessageLevel(int _messagelevel);

   /**
    * Get the current message level.
    */
   virtual int getMessageLevel() {return messagelevel;}

  private:
   int messagelevel;
};

#endif
