/**
 * ODIN overlay management middleware toolkit
 * 
 * ODIN Group, College of Computing, Georgia Institute of Technology
 *
 * For more information:
 * http://www.cc.gatech.edu/~cooperb/odin/
 * odin.gatech.edu
 *
 * You are free to use, modify and redistribute this software. 
 * However, we make no warranty for this software, and by downloading it
 * you agree not to hold use liable for anything. Do not under any
 * circumstances use this software to share copyrighted content or do
 * anything illegal. This software was designed for academic research
 * purposes and not for illegal activities. We are not responsible for
 * anything illegal you do and by downloading this software you agree
 * not to break any laws. Enjoy!
 *
 * Brian F. Cooper
 * November 21, 2005
 */


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "Message.h"
#include "IPPeerID.h"

///////////////////////////////////// Message ///////////////////////////////////

/**
 * Create a message with a single destination. Clones are made of the PeerIDs so you can delete those after creating this message if you like. 
 * The payload is not cloned and won't be deleted when the message is deleted. Subclasses, which know the type of _payload, may have different behavior. 
 */
Message::Message(PeerID * _to, PeerID * _from, int _typ, void * _payload)
{
   to=_to->clone();
   from=_from->clone();
   typ=_typ;
   payload=_payload;
}

Message::~Message()
{
   delete to;
   delete from;
}

/**
 * Get the id of the peer this message is addressed to.
 */
PeerID * Message::getToPeer()
{
   return to;
}

/**
 * Get the id of the peer that sent this message.
 */
PeerID * Message::getFromPeer()
{
   return from;
}

/**
 * Get the type of this message.
 */
int Message::getType()
{
   return typ;
}

/**
 * Get the payload of this message.
 */
void * Message::getPayload()
{
   return payload;
}

/**
 * Get the visualization string for this message.
 */
char * Message::getVisString()
{
   return NULL;
}



//////////////////////////////////// SimpleMessage ///////////////////////////////////

/**
 * Create a message with a single destination.
 */
SimpleMessage::SimpleMessage(PeerID * _to, PeerID * _from, char * _payload) : Message(_to,_from,SimpleMessageType,(void*)(new char[strlen(_payload)+1]))
{
   //what a hack! I have to pass the buffer that will contain the string to the superclass, and then retrieve it 
   //via getPayload(), and then copy the actual payload into it! This is because the superclass constructor is
   //called first, unlike Java, where I can call super() in the body of the subclass constructor.
   char * pay=(char*)getPayload();
   strcpy(pay,_payload);
}

SimpleMessage::~SimpleMessage()
{
   delete[] getStringPayload();
}
  
char * SimpleMessage::getStringPayload()
{
   return (char*)getPayload();
}
