import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.OutputStreamWriter;
import java.io.FileOutputStream;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.Writer;
import java.util.PriorityQueue;
import java.util.zip.GZIPOutputStream;

public class Merger {

    public static final String USAGE = "Usage:\n"+
        "[source_filenames]+ [destination_filename]\n";

    /**
     * Merge input files into gzipped output file.
     *
     */
    public static void mergeFiles(String output_file, String[] input_files){
        LogFile[] input_readers = new LogFile[input_files.length];
        BufferedWriter output_writer;
        int i;

        /* open input files */
        for (i=0; i<input_readers.length; i++){
            try {
                input_readers[i] = new LogFile(input_files[i]);
            } catch (FileNotFoundException e){
                input_readers[i] = null;
            }
        }

        try {
            /* open gzipped output stream as a writer */
            output_writer = new BufferedWriter(
                    new OutputStreamWriter(
                        new GZIPOutputStream(
                            new FileOutputStream(output_file))));

            /* merge inputs onto output writer */
            Merger.merge(output_writer, input_readers);
    
            /* close input and output files */
            output_writer.flush(); output_writer.close();
            for (i=0; i<input_readers.length; i++){
                if (input_readers[i] != null) { 
                    input_readers[i].close();
                }
            }
        }catch (java.io.IOException e){ 
            e.printStackTrace();
            return; 
        }
    }
        
    /**
     * Initialize the priority queue.
     */
    private static PriorityQueue<Record> initializeQueue(
            LogFile[] input_files) throws java.io.IOException {
        Record record;
        int i;
        PriorityQueue<Record> inputs = new PriorityQueue<Record>();

        /* initialize the priority queue with the first record off of each
         * file */
        for (i = 0; i < input_files.length; i++){
            if (input_files[i] == null) { continue; }

            System.out.println("Initializing input file "+i);
            record = input_files[i].next();
            if (record != null) { 
                inputs.offer(record); 
            }
        }

        return inputs;
    }

    /**
     * Process the priority queue to generate sorted output.
     */
    private static void chewQueue(Writer output_file, 
            PriorityQueue<Record> inputs) throws java.io.IOException {
        Record record = inputs.poll();

        /* do a mergesort on the files */
        while (record != null) {
            /* write the current record */
            output_file.write(record.data+"\n");
            
            /* get another record from the source we just used */
            record = record.source.next();

            if (record != null) { 
                /* load it into the priority queue */
                inputs.offer(record);
            }

            record = inputs.poll();
        }
    }

    /**
     * Merge the input logfiles onto the given writer.
     */
    public static void merge(Writer output_file, LogFile[] input_files)
        throws java.io.IOException{
        chewQueue(output_file, initializeQueue(input_files));
    }

    /**
     * Takes 2+ arguments, where the last argument is the destination filename.
     */
    public static void main(String[] args){
        String output_file;
        if (args.length < 2) {
            System.out.println(Merger.USAGE);
            return;
        }
        String[] input_files = new String[args.length - 1];
        int i;

        output_file = args[args.length - 1];

        for (i=0; i<args.length - 1; i++){
            input_files[i] = args[i];
        }

        Merger.mergeFiles(output_file, input_files);
    }
}




