
public class Record implements Comparable<Record> {
    public final String data;
    public final long timestamp;
    public final LogFile source;

    private static final String prefix = "VIS";
    
    /**
     * Construct an object representing a single record in a logfile.
     *
     * Requires that the string passed in conform to the following format:
     * 
     * Record.prefix( *)(-?[0-9]+)( )(.*)
     *
     * This is a very generous definition of a logfile record.  
     */
    public Record(LogFile source, String data){
        this.data = data;
        this.source = source;
        this.timestamp = Long.parseLong(
                data.replaceFirst(Record.prefix+"\\s*","").split(
                    "\\s+",2)[0]);
    }

    public int compareTo(Record other){
        if (this.timestamp - other.timestamp > 0) {
            return 1;
        } else if (this.timestamp - other.timestamp < 0) {
            return -1;
        } else {
            return 0;
        }
    }

    /* this is a bit of a crude speed hack. */
    public static boolean valid(String data){
        return data.matches("\\A"+Record.prefix+"\\s+[0-9]+\\s.*");
    }

}

